;*********************************************************************
; 4xD - Quad Digital Delay - Control-PIC                             *
;*********************************************************************
; (c) 2006 SM-Sound / Seb Francis                                    *
;*********************************************************************
;
; Memory map, and constant declarations
;
;*********************************************************************


;==================================================
; MISC

#DEFINE			_C	STATUS,C
#DEFINE			_Z	STATUS,Z
LSB			EQU	0x0
MSB			EQU	0x7

DIGIT1			EQU	0x0
DIGIT2			EQU	0x1
DIGIT3			EQU	0x2
LED_SEG_a		EQU	0x0
LED_SEG_f		EQU	0x1
LED_SEG_b		EQU	0x2
LED_SEG_g		EQU	0x3
LED_SEG_e		EQU	0x4
LED_SEG_d		EQU	0x5
LED_SEG_DP		EQU	0x6
LED_SEG_c		EQU	0x7


;==================================================
; Memory - bank 0

; Min			EQU	0x0020

; Current LED display
LED_DIGIT1		EQU	0x0020
LED_DIGIT2		EQU	0x0021
LED_DIGIT3		EQU	0x0022


; Parameters
MIDI_CHANNEL		EQU	0x0024	; MIDI receive/transmit channel (0-15)
DELAY_TIME_1		EQU	0x0025	; Base delay time 1 in ms:
					; 0->49 = 0.0->4.9 (0.1 step)
					; 50->74 = 5.0->9.8 (0.2 step)
					; 75->114 = 10->49 (1 step)
					; 115->189 = 50->198 (2 step)
					; 190->219 = 200->345 (5 step)
					; 220->252 = 350->670 (10 step)
DELAY_TIME_2		EQU	0x0026	; Base delay time 2 in ms: Values as above
DELAY_TIME_3		EQU	0x0027	; Base delay time 3 in ms: Values as above
DELAY_TIME_4		EQU	0x0028	; Base delay time 4 in ms: Values as above
LFO1_RATE		EQU	0x0029	; LFO1 rate: 0=slowest (stopped), 255=fastest
LFO2_RATE		EQU	0x002A	; LFO2 rate: 0=slowest (stopped), 255=fastest
LFO1_DEPTH		EQU	0x002B	; LFO1 ptp amplitude as % of maximum possible delay time modulation (0 to 200)
LFO2_DEPTH		EQU	0x002C	; LFO2 ptp amplitude as % of maximum possible delay time modulation (0 to 200)
LFO1_PHASE		EQU	0x002D	; LFO1 amplitude offset / LFO1 phase offset
					; 0=Delay 2 LFO1 amplitude -100%
					; 1=Delay 2 LFO1 amplitude -95%
					; 19=Delay 2 LFO1 amplitude -5%
					; 20=Delay 2 LFO1 amplitude matched & in phase
					; 21=Delay 2 LFO1 amplitude matched & 2 degrees out of phase
					; 110=Delay 2 LFO1 amplitude matched & 180 degrees out of phase
LFO2_PHASE		EQU	0x002E	; LFO2 amplitude offset / LFO2 phase offset
					; Values as above
LFO1_WAVE		EQU	0x002F	; LFO1 waveform: 0=sine, 1=triangle, 2=saw up, 3=saw down, 4=square
LFO2_WAVE		EQU	0x0030	; LFO2 waveform: Values as above
LFO1_SYNC		EQU	0x0031	; LFO1 sync to MIDI clock: 0=Off, 1=16, 2=8, 3=4, 4=2, 5=1, 6=3/4, 7=1/2, 8=1/3, 9=1/4, 10=1/6, 11=1/8, 12=1/12, 13=1/16
LFO2_SYNC		EQU	0x0032	; LFO2 sync to MIDI clock: Values as above
RESAMPLING_MODE		EQU	0x0033	; Resampling mode
  DELAY_12		EQU	0x0	; Bit for delay 1 & 2: 0=Off, 1=On
  DELAY_34		EQU	0x1	; Bit for delay 3 & 4: 0=Off, 1=On
  HALF_SAMPLING_RATE_12	EQU	0x2	; Set if double delay mode "dt2"
  SINGLE_INPUT_12	EQU	0x3	; Set if double delay mode "dt2."
  HALF_SAMPLING_RATE_34	EQU	0x4	; Set if double delay mode "dt2"
  SINGLE_INPUT_34	EQU	0x5	; Set if double delay mode "dt2."
DRY_MIX_1		EQU	0x0034	; Level of dry mix: 0 to 99
DRY_MIX_2		EQU	0x0035	; Level of dry mix: 0 to 99
DRY_MIX_3		EQU	0x0036	; Level of dry mix: 0 to 99
DRY_MIX_4		EQU	0x0037	; Level of dry mix: 0 to 99
FEEDBACK_1		EQU	0x0038	; Feedback level: 0 to 99
FEEDBACK_2		EQU	0x0039	; Feedback level: 0 to 99
FEEDBACK_3		EQU	0x003A	; Feedback level: 0 to 99
FEEDBACK_4		EQU	0x003B	; Feedback level: 0 to 99

; MIDI stuff
MIDI_FLAGS		EQU	0x003C
  WHICH_BYTE		EQU	0x0	; 0 - next data byte is 1st, 1 - next data byte is 2nd
  MSG_LEN		EQU	0x1	; Message length: 0=2byte 1=3byte
  SEND_MIDI_CC		EQU	0x2	; Flag indicating a MIDI CC message should be sent with new parameter value
  SET_CHANNEL_MODE	EQU	0x3	; In 'Set MIDI channel' mode (LFO Waveform button held on power up)
  ODD_VALUE		EQU	0x4	; Used internally by ProcessMIDI_ControlChange

MIDI_MSG_PROCESS_PTR	EQU	0x003D	; Pointer to next MIDI message to process
MIDI_MSG_INCOMING_PTR	EQU	0x003E	; Pointer to where to write next incoming MIDI message

MIDI_MSG1		EQU	0x003F	; Temp buffer for message byte 1
MIDI_MSG2		EQU	0x0040	; Temp buffer for message byte 2


PARAMS_CHANGED		EQU	0x0041	; Flags for when parameters have changed
  bMIDI_CHANNEL		EQU	0	; (to be checked every few seconds and written to EEPROM as necessary)
  bDELAY_TIME_1		EQU	1
  bDELAY_TIME_2		EQU	2
  bDELAY_TIME_3		EQU	3
  bDELAY_TIME_4		EQU	4
  bLFO1_RATE		EQU	5
  bLFO2_RATE		EQU	6
  bLFO1_DEPTH		EQU	7
PARAMS_CHANGED2		EQU	0x0042	; Flags for when parameters have changed
  bLFO2_DEPTH		EQU	0	; (to be checked every few seconds and written to EEPROM as necessary)
  bLFO1_PHASE		EQU	1
  bLFO2_PHASE		EQU	2
  bLFO1_WAVE		EQU	3
  bLFO2_WAVE		EQU	4
  bLFO1_SYNC		EQU	5
  bLFO2_SYNC		EQU	6
  bRESAMPLING_MODE	EQU	7
PARAMS_CHANGED3		EQU	0x0043	; Flags for when parameters have changed
  bDRY_MIX_1		EQU	0	; (to be checked every few seconds and written to EEPROM as necessary)
  bDRY_MIX_2		EQU	1
  bDRY_MIX_3		EQU	2
  bDRY_MIX_4		EQU	3
  bFEEDBACK_1		EQU	4
  bFEEDBACK_2		EQU	5
  bFEEDBACK_3		EQU	6
  bFEEDBACK_4		EQU	7

ENC_STATES		EQU	0x0044  ; Current states of rotary encoders
  DELAY_TIME_1_A	EQU	0x0
  DELAY_TIME_1_B	EQU	0x1
  DELAY_TIME_2_A	EQU	0x2
  DELAY_TIME_2_B	EQU	0x3
  DELAY_TIME_3_A	EQU	0x4
  DELAY_TIME_3_B	EQU	0x5
  DELAY_TIME_4_A	EQU	0x6
  DELAY_TIME_4_B	EQU	0x7
ENC_STATES2		EQU	0x0045  ; Current states of rotary encoders
  LFO_RATE_A		EQU	0x0
  LFO_RATE_B		EQU	0x1
  LFO_DEPTH_A		EQU	0x2
  LFO_DEPTH_B		EQU	0x3
  LFO_PHASE_A		EQU	0x4
  LFO_PHASE_B		EQU	0x5
  LFO_WAVE_A		EQU	0x6

EEPROM_DATA_WRITE_L	EQU	0x0046	; When this gets incremented to >= 0x800 by TMR0 then check if anything needs writing to EEPROM
EEPROM_DATA_WRITE_H	EQU	0x0047

DB_DELAY_TIME_1		EQU	0x0048	; Time count for debouncing rotary encoder switching
DB_DELAY_TIME_2		EQU	0x0049	; Time count for debouncing rotary encoder switching
DB_DELAY_TIME_3		EQU	0x004A	; Time count for debouncing rotary encoder switching
DB_DELAY_TIME_4		EQU	0x004B	; Time count for debouncing rotary encoder switching
DB_LFO_RATE		EQU	0x004C	; Time count for debouncing rotary encoder switching
DB_LFO_DEPTH		EQU	0x004D	; Time count for debouncing rotary encoder switching
DB_LFO_PHASE		EQU	0x004E	; Time count for debouncing rotary encoder switching
DB_LFO_WAVE		EQU	0x004F	; Time count for debouncing push button switch

  
MIDI_MSG_BUFFER		EQU	0x0050	; start of MIDI message ring buffer
;				0x0051	; (needs to be at this address for PointToNextIncomingBufferByte macro to work)
;				0x0052
;				0x0053
;				0x0054
;				0x0055
;				0x0056
;				0x0057
;				0x0058
;				0x0059
;				0x005A
;				0x005B
;				0x005C
;				0x005D
;				0x005E
MIDI_MSG_BUFFER_END	EQU	0x005F	; end of MIDI message ring buffer


; Max			EQU	0x005F


; Stuff for fixed point maths routines
; 0x0060-0x006F in bank 0  - used in main loop
; 0x00E0-0x00EF in bank 1  - used by interupts, so don't call maths routines in bank 1 from main loop!
; 0x0160-0x016F in bank 2  - memory used for settings, don't call maths routines in bank 2!
; 0x01E0-0x01EF in bank 3  - 0x1EB-0x1EF used by ICD, don't call maths routines in bank 3!
AARGB2			EQU	0x0060		; argument A, LSB
AARGB1			EQU	0x0061		; argument A
AARGB0			EQU	0x0062		; argument A, MSB
BARGB1			EQU	0x0063		; argument B, LSB
BARGB0			EQU	0x0064		; argument B, MSB


LED_INTERRUPT_COUNT_L	EQU	0x0065	; Extra bits to increase length of LED_INTERRUPT_COUNT
LED_INTERRUPT_COUNT	EQU	0x0066	; Count of LED interrupts (when this reaches 0, turn LED off)


; MIDI Sync variables
MIDI_CLOCK_PERIOD1_L	EQU	0x0067		; Number of clock ticks per sync period (LFO1, low byte)
MIDI_CLOCK_PERIOD1_H	EQU	0x0068		; Number of clock ticks per sync period (LFO1, low byte)
MIDI_CLOCK_PERIOD2_L	EQU	0x0069		; Number of clock ticks per sync period (LFO1, low byte)
MIDI_CLOCK_PERIOD2_H	EQU	0x006A		; Number of clock ticks per sync period (LFO1, low byte)
MIDI_CLOCK_COUNT1_L	EQU	0x006B		; Count of received MIDI clock messages (LFO1, low byte)
MIDI_CLOCK_COUNT1_H	EQU	0x006C		; Count of received MIDI clock messages (LFO1, high byte)
MIDI_CLOCK_COUNT2_L	EQU	0x006D		; Count of received MIDI clock messages (LFO2, low byte)
MIDI_CLOCK_COUNT2_H	EQU	0x006E		; Count of received MIDI clock messages (LFO2, high byte)
TIMER2_H		EQU	0x006F		; High byte of timer2 counter


; Memory accessible in all banks

RESERVED		EQU	0x0070		; Reserved for ICD

W_TEMP        		EQU     0x0071        	; variable used for context saving
STATUS_TEMP		EQU	0x0072		; variable used for context saving
PCLATH_TEMP		EQU	0x0073		; variable used for context saving
FSR_TEMP		EQU	0x0074		; variable used for context saving

; Temp vars for use in interrupts (or during initialisation)
TEMP1			EQU	0x0075
TEMP2			EQU	0x0076
WAITER			EQU	0x0077

; Temp vars for use in main loop (or during initialisation)
MTEMP1			EQU	0x0078
MTEMP2			EQU	0x0079
MTEMP3			EQU	0x007A
MTEMP4			EQU	0x007B
MWAITER			EQU	0x007C


; Other variables that didn't fit in normal bank 0 space
FLAGS			EQU	0x007D	; Various flags
  CURRENT_DIGIT_2	EQU	0x0	; If set, current LED digit to update is digit 2
  CURRENT_DIGIT_3	EQU	0x1	; If set, current LED digit to update is digit 3
  INTERRUPTS_OFF	EQU	0x2	; Used internally by EEPROM reading and writing subroutines
  READ_CONTROLS		EQU	0x3	; Set by timer 1 every 0.25ms indicating controls should be read
  SHOW_TENS		EQU	0x4	; Used internally by DisplayDelayTime and DisplayNumber
  DONT_CHANGE_WAVE	EQU	0x5	; Set when a control is changed while WAVE/FN button is held

SPI_MSG			EQU	0x007E	; Temporary register for SPI message byte



; Max			EQU	0x007F


;==================================================
; Memory - bank 1

; Min			EQU	0x00A0


; Max			EQU	0x00DF

; Stuff for fixed point maths routines
; 0x0060-0x006F in bank 0 - used in main loop
; 0x00E0-0x00EF in bank 1 - used by interupts


;==================================================
; Memory - bank 2

; Min			EQU	0x0110


; Max			EQU	0x016F

;==================================================
; Memory - bank 3

; Min			EQU	0x0190


; Max			EQU	0x01EA

; 0x01EB-0x01EF reserved for ICD

;==================================================
; EEPROM

; Min			EQU	0x00

EE_MIDI_CHANNEL		EQU	0x00	; MIDI receive/transmit channel (0-15 or 0xFF indicating EEPROM contents are invalid)

; Effect parameters [see main RAM for value definitions]
EE_DELAY_TIME_1		EQU	0x01
EE_DELAY_TIME_2		EQU	0x02
EE_DELAY_TIME_3		EQU	0x03
EE_DELAY_TIME_4		EQU	0x04
EE_LFO1_RATE		EQU	0x05
EE_LFO2_RATE		EQU	0x06
EE_LFO1_DEPTH		EQU	0x07
EE_LFO2_DEPTH		EQU	0x08
EE_LFO1_PHASE		EQU	0x09
EE_LFO2_PHASE		EQU	0x0A
EE_LFO1_WAVE		EQU	0x0B
EE_LFO2_WAVE		EQU	0x0C
EE_LFO1_SYNC		EQU	0x0D
EE_LFO2_SYNC		EQU	0x0E
EE_RESAMPLING_MODE	EQU	0x0F
EE_DRY_MIX_1		EQU	0x10
EE_DRY_MIX_2		EQU	0x11
EE_DRY_MIX_3		EQU	0x12
EE_DRY_MIX_4		EQU	0x13
EE_FEEDBACK_1		EQU	0x14
EE_FEEDBACK_2		EQU	0x15
EE_FEEDBACK_3		EQU	0x16
EE_FEEDBACK_4		EQU	0x17

; Max			EQU	0xFF
