;*********************************************************************
; 4xD - Quad Digital Delay - Control-PIC                             *
;*********************************************************************
; (c) 2006 SM-Sound / Seb Francis                                    *
;*********************************************************************
;
; Main code
;
;*********************************************************************
;                                                               
;    Connections:
;
;    RA0   <- Physical input device (see below)
;    RA1   <- Physical input device (see below)
;    RA2   <- Physical input device (see below)
;    RA3   <- Physical input device (see below)
;    RA4   <- Physical input device (see below)
;    RA5   <- Physical input device (see below)
;
;    RB0   <- Physical input device (see below)
;    RB1   <- Physical input device (see below)
;    RB2   <- Physical input device (see below)
;    RB3   <- Physical input device (see below)
;    RB4   <- Physical input device (see below)
;    RB5   <- Physical input device (see below)
;    RB6   <- Physical input device (see below)   *and*   <-> PGC
;    RB7   <-> PGD
;
;    RC0   -> LED AN1
;    RC1   -> LED AN2
;    RC2   -> LED AN3
;    RC3   -> SPI SCK
;    RC4   <- SPI SDI (not used)
;    RC5   -> SPI SDO
;    RC6   -> MIDI Out
;    RC7   <- MIDI In
;
;    RD0   -> LED c
;    RD1   -> LED DP
;    RD2   -> LED e
;    RD3   -> LED d
;    RD4   -> LED g
;    RD5   -> LED b
;    RD6   -> LED f
;    RD7   -> LED a
;
;    RE0   <- Physical input device (see below)
;    RE1   <- Physical input device (see below)
;    RE2   <- Physical input device (see below)
;
;
;    Clock: 12.500 MHz -> 3.125 MIPS
;
;
;*********************************************************************

; #define SWAP_ENCODERS		; Uncomment to swap encoders (if need to change LFO Depth while ICD debugging)

; Define which port pins each physical input device is connected to
	#define	PORT_DELAY_TIME_1_A	PORTB,0
	#define	PORT_DELAY_TIME_1_B	PORTB,1
	#define PORT_DELAY_TIME_2_A	PORTA,0
	#define PORT_DELAY_TIME_2_B	PORTB,2
	#define PORT_DELAY_TIME_3_A	PORTA,5
	#define PORT_DELAY_TIME_3_B	PORTA,1
	#define PORT_DELAY_TIME_4_A	PORTA,4
	#define PORT_DELAY_TIME_4_B	PORTE,1
	#define PORT_LFO_PHASE_A	PORTE,2
	#define PORT_LFO_PHASE_B	PORTE,0
#ifdef SWAP_ENCODERS
	#define PORT_LFO_RATE_A		PORTB,6
	#define PORT_LFO_RATE_B		PORTA,2
	#define PORT_LFO_DEPTH_A	PORTB,5
	#define PORT_LFO_DEPTH_B	PORTB,3
#else
	#define PORT_LFO_RATE_A		PORTB,5
	#define PORT_LFO_RATE_B		PORTB,3
	#define PORT_LFO_DEPTH_A	PORTB,6
	#define PORT_LFO_DEPTH_B	PORTA,2
#endif
	#define PORT_LFO_WAVE		PORTA,3
	#define PORT_LFO_SELECT		PORTB,4

;*********************************************************************


	list      P=PIC16F877A	; List directive to define processor
	#include <P16F877A.INC>
	#include <const.inc>	; Constants + Memory map
	#include <misc.inc>	; Various macros
	

; '__CONFIG' directive is used to embed configuration data within .asm file.
; The labels following the directive are located in the respective .inc file.
; See respective data sheet for additional information on configuration word.
	
	__CONFIG _CP_OFF & _DEBUG_OFF & _WRT_OFF & _CPD_OFF & _LVP_OFF & _BODEN_OFF & _PWRTE_ON & _WDT_OFF & _HS_OSC


;**********************************************************************


	org     0x0000		; Processor reset vector

	nop			; Must have nop here for ICD to work
	
	clrf    PCLATH		; Ensure page bits are cleared
	goto    Init		; Go to beginning of program


	org     0x0004		; Interrupt vector location
	
	movwf   W_TEMP		; Save W, STATUS, PCLATH and FSR
	swapf	STATUS,w	; Use swapf, not movf so not to disturb Z flag
	movwf	STATUS_TEMP
	movfw   PCLATH
	movwf	PCLATH_TEMP
	movfw	FSR
	movwf	FSR_TEMP
	

; Call the interupt handlers

	movlw	high UpdateLEDs	; Point PCLATH to program bank 1
	movwf	PCLATH

	clrf	STATUS		; RAM bank 0, IRP 0
	btfsc	INTCON,TMR0IF
	call	UpdateLEDs

	clrf	STATUS		; RAM bank 0, IRP 0
	btfsc	PIR1,TMR1IF
	call	Timer1Int

	clrf	STATUS		; RAM bank 0, IRP 0
	btfsc	PIR1,TMR2IF
	call	Timer2Int

	clrf	STATUS		; RAM bank 0, IRP 0
	btfsc	PIR1,RCIF
	call	MIDIByteReceived

; --
	movfw	FSR_TEMP	; Restore W, STATUS, PCLATH and FSR
	movwf	FSR
	movfw	PCLATH_TEMP
	movwf	PCLATH
	swapf	STATUS_TEMP,w	; Use swapf, not movf so not to disturb Z flag
	movwf	STATUS
	swapf	W_TEMP,f	; Use swapf, not movf so not to disturb Z flag
	swapf	W_TEMP,w	; Swap nibbles again to get them correct
InterruptEnd
	retfie			; Return from interrupt


;**********************************************************************


Init

; Bank 3 Setup
	bsf	STATUS,RP0	; Bank 3
	bsf	STATUS,RP1
	
	clrf	EECON1		; Access EEPROM data memory, EEPROM write disable
	
	
; Bank 0 Setup
	clrf	STATUS		; RAM bank 0, IRP 0
	
	clrf	PORTA		; Set all outputs to off initially
	clrf	PORTB
	clrf	PORTC
	clrf	PORTD
	clrf	PORTE
	
	movlw	b'01100000'	; Global disable interrupts, enable interrupts: peripheral, TMR0
	movwf	INTCON
	
	clrf	PIR1		; Clear interrupt flags
	clrf	PIR2
	
	movlw	b'00000001'	; TMR1: prescale 1:1, oscillator disabled, internal clock, timer running
	movwf	T1CON
	
	movlw	b'00000110'	; TMR2: postscale 1:1, timing running, prescale 1:16 (increment at 195.3125 kHz = sample clock * 4)
	movwf	T2CON
	
	movlw	b'00100001'	; SSP: Enabled, clock idle state = low, SPI master mode, clock = FOSC/16
	movwf	SSPCON
	
	clrf	CCP1CON		; CCP1: off
	clrf	CCP2CON		; CCP2: off
	
	movlw	b'10000000'	; USART: enabled, 8 bit reception, reception (CREN) disabled, address detection disabled
	movwf	RCSTA
	
	movlw	b'00000000'	; A/D converter off
	movwf	ADCON0
	
	
; Initialise variables in bank 0
	
	; Set all LED digits blank
	movlw	0xFF
	movwf	LED_DIGIT1
	movwf	LED_DIGIT2
	movwf	LED_DIGIT3
	
	; Clear flags
	clrf	FLAGS
	
	; Set LED display counter to off
	clrf	LED_INTERRUPT_COUNT
	clrf	LED_INTERRUPT_COUNT_L
	
	clrf	MIDI_FLAGS	; Clear MIDI flags
	clrf	MIDI_MSG1	; Clear MIDI status buffer
	
	movlw	low MIDI_MSG_BUFFER	; Set pointers to MIDI ring buffer
	movwf	MIDI_MSG_PROCESS_PTR
	movwf	MIDI_MSG_INCOMING_PTR
	
	clrf	EEPROM_DATA_WRITE_L
	clrf	EEPROM_DATA_WRITE_H
	clrf	PARAMS_CHANGED
	clrf	PARAMS_CHANGED2
	clrf	PARAMS_CHANGED3
	
	clrf	DB_DELAY_TIME_1
	clrf	DB_DELAY_TIME_2
	clrf	DB_DELAY_TIME_3
	clrf	DB_DELAY_TIME_4
	clrf	DB_LFO_RATE
	clrf	DB_LFO_DEPTH
	clrf	DB_LFO_PHASE
	clrf	DB_LFO_WAVE
	
	clrf	MIDI_CLOCK_COUNT1_L
	clrf	MIDI_CLOCK_COUNT1_H
	clrf	MIDI_CLOCK_COUNT2_L
	clrf	MIDI_CLOCK_COUNT2_H
	clrf	TIMER2_H
	
	
; Load settings from EEPROM

	; Read MIDI_CHANNEL
	movlw	EE_MIDI_CHANNEL
	movwf	TEMP1
	call	ReadByteFromEEPROM_Int
	movwf	MIDI_CHANNEL
	
	; If MIDI_CHANNEL != 0xFF, then EEPROM is valid
	incf	MIDI_CHANNEL,w
	jnz	EEPROM_Valid
	
	; Write default values to RAM & EEPROM
	clrf	TEMP2
	movlw	EE_MIDI_CHANNEL
	movfw	TEMP1
	call	WriteByteToEEPROM_Int
	clrf	MIDI_CHANNEL
	
	incf	TEMP1
	call	WriteByteToEEPROM_Int
	clrf	DELAY_TIME_1
	
	incf	TEMP1
	call	WriteByteToEEPROM_Int
	clrf	DELAY_TIME_2
	
	incf	TEMP1
	call	WriteByteToEEPROM_Int
	clrf	DELAY_TIME_3
	
	incf	TEMP1
	call	WriteByteToEEPROM_Int
	clrf	DELAY_TIME_4
	
	incf	TEMP1
	call	WriteByteToEEPROM_Int
	clrf	LFO1_RATE
	
	incf	TEMP1
	call	WriteByteToEEPROM_Int
	clrf	LFO2_RATE
	
	incf	TEMP1
	call	WriteByteToEEPROM_Int
	clrf	LFO1_DEPTH
	
	incf	TEMP1
	call	WriteByteToEEPROM_Int
	clrf	LFO2_DEPTH
	
	movlw	D'20'
	movwf	TEMP2
	movwf	LFO1_PHASE
	movwf	LFO2_PHASE
	incf	TEMP1
	call	WriteByteToEEPROM_Int
	incf	TEMP1
	call	WriteByteToEEPROM_Int
	clrf	TEMP2
	
	incf	TEMP1
	call	WriteByteToEEPROM_Int
	clrf	LFO1_WAVE
	
	incf	TEMP1
	call	WriteByteToEEPROM_Int
	clrf	LFO2_WAVE
	
	incf	TEMP1
	call	WriteByteToEEPROM_Int
	clrf	LFO1_SYNC
	
	incf	TEMP1
	call	WriteByteToEEPROM_Int
	clrf	LFO2_SYNC
	
	movlw	b'00000011'
	movwf	TEMP2
	movwf	RESAMPLING_MODE
	incf	TEMP1
	call	WriteByteToEEPROM_Int
	clrf	TEMP2
	
	incf	TEMP1
	call	WriteByteToEEPROM_Int
	clrf	DRY_MIX_1
	
	incf	TEMP1
	call	WriteByteToEEPROM_Int
	clrf	DRY_MIX_2
	
	incf	TEMP1
	call	WriteByteToEEPROM_Int
	clrf	DRY_MIX_3
	
	incf	TEMP1
	call	WriteByteToEEPROM_Int
	clrf	DRY_MIX_4
	
	incf	TEMP1
	call	WriteByteToEEPROM_Int
	clrf	FEEDBACK_1
	
	incf	TEMP1
	call	WriteByteToEEPROM_Int
	clrf	FEEDBACK_2
	
	incf	TEMP1
	call	WriteByteToEEPROM_Int
	clrf	FEEDBACK_3
	
	incf	TEMP1
	call	WriteByteToEEPROM_Int
	clrf	FEEDBACK_4
	
	
EEPROM_Valid
	; Load values from EEPROM
	movlw	EE_DELAY_TIME_1
	movwf	TEMP1
	call	ReadByteFromEEPROM_Int
	movwf	DELAY_TIME_1
	
	incf	TEMP1
	call	ReadByteFromEEPROM_Int
	movwf	DELAY_TIME_2
	
	incf	TEMP1
	call	ReadByteFromEEPROM_Int
	movwf	DELAY_TIME_3
	
	incf	TEMP1
	call	ReadByteFromEEPROM_Int
	movwf	DELAY_TIME_4
	
	incf	TEMP1
	call	ReadByteFromEEPROM_Int
	movwf	LFO1_RATE
	
	incf	TEMP1
	call	ReadByteFromEEPROM_Int
	movwf	LFO2_RATE
	
	incf	TEMP1
	call	ReadByteFromEEPROM_Int
	movwf	LFO1_DEPTH
	
	incf	TEMP1
	call	ReadByteFromEEPROM_Int
	movwf	LFO2_DEPTH
	
	incf	TEMP1
	call	ReadByteFromEEPROM_Int
	movwf	LFO1_PHASE
	
	incf	TEMP1
	call	ReadByteFromEEPROM_Int
	movwf	LFO2_PHASE
	
	incf	TEMP1
	call	ReadByteFromEEPROM_Int
	movwf	LFO1_WAVE
	
	incf	TEMP1
	call	ReadByteFromEEPROM_Int
	movwf	LFO2_WAVE
	
	incf	TEMP1
	call	ReadByteFromEEPROM_Int
	movwf	LFO1_SYNC
	
	incf	TEMP1
	call	ReadByteFromEEPROM_Int
	movwf	LFO2_SYNC
	
	incf	TEMP1
	call	ReadByteFromEEPROM_Int
	movwf	RESAMPLING_MODE
	
	incf	TEMP1
	call	ReadByteFromEEPROM_Int
	movwf	DRY_MIX_1
	
	incf	TEMP1
	call	ReadByteFromEEPROM_Int
	movwf	DRY_MIX_2
	
	incf	TEMP1
	call	ReadByteFromEEPROM_Int
	movwf	DRY_MIX_3
	
	incf	TEMP1
	call	ReadByteFromEEPROM_Int
	movwf	DRY_MIX_4
	
	incf	TEMP1
	call	ReadByteFromEEPROM_Int
	movwf	FEEDBACK_1
	
	incf	TEMP1
	call	ReadByteFromEEPROM_Int
	movwf	FEEDBACK_2
	
	incf	TEMP1
	call	ReadByteFromEEPROM_Int
	movwf	FEEDBACK_3
	
	incf	TEMP1
	call	ReadByteFromEEPROM_Int
	movwf	FEEDBACK_4
	
	
	
; Bank 1 Setup
	bsf	STATUS,RP0	; RAM bank 1
	
	movlw	b'00000100'	; PORTB pullups enabled, Internal TMR0 clock, Pre-scaler assigned to TMR0, 1:32 (interrupt every 2.62144ms)
	movwf	OPTION_REG
	
	movlw	b'00111111'	; RA0-5 inputs
	movwf	TRISA
	
	movlw	b'11111111'	; RB0-7 inputs
	movwf	TRISB
	
	movlw	b'10010000'	; RC4 SPI input, RC7 USART input, all else outputs
	movwf	TRISC
	
	clrf	TRISD		; PORTD all outputs
	
	movlw	b'00000111'	; RE0-2 inputs
	movwf	TRISE
	
	movlw	b'00100011'	; Peripheral interrupts 1: USART receive on, TMR1 on, TMR2 on, everything else off
	movwf	PIE1
	
	clrf	PIE2		; Peripheral interrupts 2: All off
	
	clrf	SSPCON2		; SSP settings (not used for SPI mode)
	
	movlw	0xFF		; TMR2 period register set for 256 increments per cycle
	movwf	PR2
	
	movlw	b'00000000'	; SPI Input data sampled at middle of data output time, Transmit on transition from idle to active clock state
	movwf	SSPSTAT
	
	movlw	b'00100100'	; USART: 8 bit transmission, transmit enabled, asynchronous mode, high baud rate
	movwf	TXSTA
	
	movlw	D'24'		; USART: Baud rate 31,250
	movwf	SPBRG
	
	movlw	b'00000111'	; Comparators off
	movwf	CMCON
	clrf	CVRCON		; Comparator voltage reference off
	
	movlw	b'00000110'	; Disable all A/D inputs (PORTA digital I/O)
	movwf	ADCON1


; Bank 2 Setup
; 	bcf	STATUS,RP0	; RAM bank 2
; 	bsf	STATUS,RP1
	
	; Nothing to do in bank 2


	
;--
	clrf	STATUS		; RAM bank 0, IRP 0
	
	
; Power up LED sequence
	; All segments off
	movlw	0xFF
	movwf	PORTD
	
	bsf	PORTC,DIGIT1
	call	LoopDigit
	bcf	PORTC,DIGIT1
	bsf	PORTC,DIGIT2
	call	LoopDigit
	bcf	PORTC,DIGIT2
	bsf	PORTC,DIGIT3
	call	LoopDigit
	bcf	PORTC,DIGIT3

	goto	PowerUpComplete
	
LoopDigit
	bcf	PORTD,LED_SEG_d
	call	Pause
	bsf	PORTD,LED_SEG_d
	bcf	PORTD,LED_SEG_c
	call	Pause
	bsf	PORTD,LED_SEG_c
	bcf	PORTD,LED_SEG_b
	call	Pause
	bsf	PORTD,LED_SEG_b
	bcf	PORTD,LED_SEG_a
	call	Pause
	bsf	PORTD,LED_SEG_a
	bcf	PORTD,LED_SEG_f
	call	Pause
	bsf	PORTD,LED_SEG_f
	bcf	PORTD,LED_SEG_e
	call	Pause
	bsf	PORTD,LED_SEG_e
	bcf	PORTD,LED_SEG_d
	call	Pause
	bsf	PORTD,LED_SEG_d
	return
Pause
	wait_ms	38	; Wait 60.8ms for a 12.5 MHz clock
	wait_ms	38	; Wait 60.8ms for a 12.5 MHz clock
	wait_ms	38	; Wait 60.8ms for a 12.5 MHz clock
	wait_ms	38	; Wait 60.8ms for a 12.5 MHz clock
	return


PowerUpComplete
	
	; Read current encoder states
	clrf	ENC_STATES
	clrf	ENC_STATES2
	btfsc	PORT_DELAY_TIME_1_A
	bsf	ENC_STATES,DELAY_TIME_1_A
	btfsc	PORT_DELAY_TIME_1_B
	bsf	ENC_STATES,DELAY_TIME_1_B
	btfsc	PORT_DELAY_TIME_2_A
	bsf	ENC_STATES,DELAY_TIME_2_A
	btfsc	PORT_DELAY_TIME_2_B
	bsf	ENC_STATES,DELAY_TIME_2_B
	btfsc	PORT_DELAY_TIME_3_A
	bsf	ENC_STATES,DELAY_TIME_3_A
	btfsc	PORT_DELAY_TIME_3_B
	bsf	ENC_STATES,DELAY_TIME_3_B
	btfsc	PORT_DELAY_TIME_4_A
	bsf	ENC_STATES,DELAY_TIME_4_A
	btfsc	PORT_DELAY_TIME_4_B
	bsf	ENC_STATES,DELAY_TIME_4_B
	btfsc	PORT_LFO_RATE_A
	bsf	ENC_STATES2,LFO_RATE_A
	btfsc	PORT_LFO_RATE_B
	bsf	ENC_STATES2,LFO_RATE_B
	btfsc	PORT_LFO_DEPTH_A
	bsf	ENC_STATES2,LFO_DEPTH_A
	btfsc	PORT_LFO_DEPTH_B
	bsf	ENC_STATES2,LFO_DEPTH_B
	btfsc	PORT_LFO_PHASE_A
	bsf	ENC_STATES2,LFO_PHASE_A
	btfsc	PORT_LFO_PHASE_B
	bsf	ENC_STATES2,LFO_PHASE_B
	btfsc	PORT_LFO_WAVE
	bsf	ENC_STATES2,LFO_WAVE_A
	
	btfss	PORT_LFO_WAVE
	bsf	MIDI_FLAGS,SET_CHANNEL_MODE
	
	
	; Send current parameter values to DSP PICs
	bcf	MIDI_FLAGS,SEND_MIDI_CC	; Clear flag so no MIDI messages are sent
	
	bsf	PCLATH, 4		; Program bank 2
	
	movfw	DELAY_TIME_1
	movwf	MTEMP4
	call	Check_DELAY_TIME_1
	
	movfw	DELAY_TIME_2
	movwf	MTEMP4
	call	Check_DELAY_TIME_2
	
	movfw	DELAY_TIME_3
	movwf	MTEMP4
	call	Check_DELAY_TIME_3
	
	movfw	DELAY_TIME_4
	movwf	MTEMP4
	call	Check_DELAY_TIME_4
	
	movfw	LFO1_RATE
	movwf	MTEMP4
	call	Check_LFO1_RATE
	
	movfw	LFO2_RATE
	movwf	MTEMP4
	call	Check_LFO2_RATE
	
	movfw	LFO1_DEPTH
	movwf	MTEMP4
	call	Check_LFO1_DEPTH
	
	movfw	LFO2_DEPTH
	movwf	MTEMP4
	call	Check_LFO2_DEPTH
	
	movfw	LFO1_PHASE
	movwf	MTEMP4
	call	Check_LFO1_PHASE
	
	movfw	LFO2_PHASE
	movwf	MTEMP4
	call	Check_LFO2_PHASE
	
	movfw	LFO1_WAVE
	movwf	MTEMP4
	call	Change_LFO1_WAVE
	
	movfw	LFO2_WAVE
	movwf	MTEMP4
	call	Change_LFO2_WAVE
	
	movfw	LFO1_SYNC
	movwf	MTEMP4
	call	Check_LFO1_SYNC
	
	movfw	LFO2_SYNC
	movwf	MTEMP4
	call	Check_LFO2_SYNC
	
	clrf	MTEMP4
	btfsc	RESAMPLING_MODE,DELAY_12
	decf	MTEMP4
	call	Check_RESAMPLING_MODE_12
	
	clrf	MTEMP4
	btfsc	RESAMPLING_MODE,DELAY_34
	decf	MTEMP4
	call	Check_RESAMPLING_MODE_34
	
	movfw	DRY_MIX_1
	movwf	MTEMP4
	call	Check_DRY_MIX_1
	
	movfw	DRY_MIX_2
	movwf	MTEMP4
	call	Check_DRY_MIX_2
	
	movfw	DRY_MIX_3
	movwf	MTEMP4
	call	Check_DRY_MIX_3
	
	movfw	DRY_MIX_4
	movwf	MTEMP4
	call	Check_DRY_MIX_4
	
	movfw	FEEDBACK_1
	movwf	MTEMP4
	call	Check_FEEDBACK_1
	
	movfw	FEEDBACK_2
	movwf	MTEMP4
	call	Check_FEEDBACK_2
	
	movfw	FEEDBACK_3
	movwf	MTEMP4
	call	Check_FEEDBACK_3
	
	movfw	FEEDBACK_4
	movwf	MTEMP4
	call	Check_FEEDBACK_4
	
	clrf	LED_INTERRUPT_COUNT	; Turn off display
	
	; Enable audio
	; SPI_MSG - Chip select & Message type nybble: c000mmmm
	; AARGB1  - Value to transmit byte
	movlw	1
	movwf	AARGB1
	movlw	b'00001111'
	movwf	SPI_MSG
	call	TransmitSPI
	movlw	b'10001111'
	movwf	SPI_MSG
	call	TransmitSPI
	
	bcf	PCLATH, 4		; Program bank 0


	bsf	RCSTA,CREN	; Enable MIDI receive
	bcf	INTCON,TMR0IF	; Reset TMR0 flag
	bsf	INTCON,GIE	; Enable interrupts


;#######################################################################

; RAM Bank 0 !

; ----------------------------------------------------------------------

	btfss	MIDI_FLAGS,SET_CHANNEL_MODE
	goto	Main
	
	bsf	FLAGS,DONT_CHANGE_WAVE	; Set flag
SetChannelLoop
	; Display 'Cnn'
	
	movlw	high LEDNumbers			; Set PCLATH for jump tables (and point at program bank 2)
	movwf	PCLATH
	
	movlw	low (LEDText_C - LEDText_Base)	; Write 'C'
	call	GetLEDNumber
	movwf	LED_DIGIT1
	
	incf	MIDI_CHANNEL,w			; Convert MIDI_CHANNEL to BCD
	call	ConvertToBcd
	movfw	AARGB1				; and display on LED
	movlw	0xFF		; If 0, digit off
	bcf	PCLATH, 4			; Program bank 0
	jz	SetChannelLoop_1
	bsf	PCLATH, 4			; Program bank 2
	movfw	AARGB1
	call	GetLEDNumber
SetChannelLoop_1
	bsf	PCLATH, 4			; Program bank 2
	movwf	LED_DIGIT2
	movfw	AARGB2
	call	GetLEDNumber
	movwf	LED_DIGIT3
	
	clrf	LED_INTERRUPT_COUNT	; Turn on display
	incf	LED_INTERRUPT_COUNT
	
	bcf	PCLATH, 4		; Program bank 0
	
	btfsc	FLAGS,READ_CONTROLS	; Check if time to read control (flag set by timer 1 every 0.25ms)
	call	ReadChannel

	btfss	PORT_LFO_WAVE		; When LFO Wave button released, exit 'Set MIDI channel' Mode
	goto	SetChannelLoop
	bcf	MIDI_FLAGS,SET_CHANNEL_MODE
	
	clrf	LED_INTERRUPT_COUNT	; Turn off display
	decf	LED_INTERRUPT_COUNT

Main
	btfsc	EEPROM_DATA_WRITE_H,3	; Once every 5.3 seconds check if anything needs writing to EEPROM
	goto	CheckWriteToEEPROM
	
	btfsc	FLAGS,READ_CONTROLS	; Check if time to read controls (flag set by timer 1 every 0.25ms)
	call	ReadControls
	
	movfw	MIDI_MSG_PROCESS_PTR	; Check if MIDI buffer has any messages
	subwf	MIDI_MSG_INCOMING_PTR,w
	jz	Main
	
Main_GotMIDI
	; Got some MIDI to process ..
	bsf	PCLATH, 4		; Program bank 2
	call	ProcessMIDI
	bcf	PCLATH, 4		; Program bank 0
	
	goto	Main


;-----------------------------------------------------------------------------------------


CheckWriteToEEPROM

	; Test bits of PARAMS_CHANGED
	
	clrf	MTEMP4		; Set parameter counter to 0
	clrf	MTEMP3		; Set bit 0
	bsf	MTEMP3,0

CheckWriteToEEPROM_Loop
	movfw	MTEMP3			; Load bitmask into W
	jz	CheckWriteToEEPROM2	; If 0 then has rotated through all bits, so jump to next bit

	andwf	PARAMS_CHANGED,w	; Test specific bit of PARAMS_CHANGED
	jnz	WriteChangedParam

ChangedParamWritten
	movfw	MIDI_MSG_PROCESS_PTR	; Check if MIDI buffer has any messages
	subwf	MIDI_MSG_INCOMING_PTR,w
	jnz	Main_GotMIDI

	incf	MTEMP4		; Increment parameter counter
	bcf	STATUS,C	; Move to next bit
	rlf	MTEMP3

	goto	CheckWriteToEEPROM_Loop
		
	
	; Now test bits of PARAMS_CHANGED2
	
CheckWriteToEEPROM2
	clrf	MTEMP4		; Set parameter counter to 0
	bsf	MTEMP3,0	; Set bit 0

CheckWriteToEEPROM2_Loop
	movfw	MTEMP3			; Load bitmask into W
	jz	CheckWriteToEEPROM3	; If 0 then has rotated through all bits, so jump to next bit

	andwf	PARAMS_CHANGED2,w	; Test specific bit of PARAMS_CHANGED2
	jnz	WriteChangedParam2

ChangedParamWritten2
	movfw	MIDI_MSG_PROCESS_PTR	; Check if MIDI buffer has any messages
	subwf	MIDI_MSG_INCOMING_PTR,w
	jnz	Main_GotMIDI

	incf	MTEMP4		; Increment parameter counter
	bcf	STATUS,C	; Move to next bit
	rlf	MTEMP3

	goto	CheckWriteToEEPROM2_Loop
		
	
	; Now test bits of PARAMS_CHANGED3
	
CheckWriteToEEPROM3
	clrf	MTEMP4		; Set parameter counter to 0
	bsf	MTEMP3,0	; Set bit 0

CheckWriteToEEPROM3_Loop
	movfw	MTEMP3			; Load bitmask into W
	jz	CheckWriteToEEPROM_End	; If 0 then has rotated through all bits, so jump to next bit

	andwf	PARAMS_CHANGED3,w	; Test specific bit of PARAMS_CHANGED3
	jnz	WriteChangedParam3

ChangedParamWritten3
	movfw	MIDI_MSG_PROCESS_PTR	; Check if MIDI buffer has any messages
	subwf	MIDI_MSG_INCOMING_PTR,w
	jnz	Main_GotMIDI

	incf	MTEMP4		; Increment parameter counter
	bcf	STATUS,C	; Move to next bit
	rlf	MTEMP3

	goto	CheckWriteToEEPROM3_Loop

	
CheckWriteToEEPROM_End

	clrf	EEPROM_DATA_WRITE_L	; Reset counter and return to main loop
	clrf	EEPROM_DATA_WRITE_H
	goto	Main
	
	
WriteChangedParam
	; Write changed param value to EEPROM
	movlw	EE_MIDI_CHANNEL
	addwf	MTEMP4,w
	movwf	MTEMP1
	movlw	MIDI_CHANNEL
	addwf	MTEMP4,w
	movwf	FSR
	movfw	INDF
	movwf	MTEMP2
	call	WriteByteToEEPROM_Main
	
	; Clear bit of PARAMS_CHANGED
	movfw	MTEMP3
	xorwf	PARAMS_CHANGED
	
	goto	ChangedParamWritten
	
WriteChangedParam2
	; Write changed param value to EEPROM
	movlw	EE_LFO2_DEPTH
	addwf	MTEMP4,w
	movwf	MTEMP1
	movlw	LFO2_DEPTH
	addwf	MTEMP4,w
	movwf	FSR
	movfw	INDF
	movwf	MTEMP2
	call	WriteByteToEEPROM_Main
	
	; Clear bit of PARAMS_CHANGED2
	movfw	MTEMP3
	xorwf	PARAMS_CHANGED2
	
	goto	ChangedParamWritten2
	
WriteChangedParam3
	; Write changed param value to EEPROM
	movlw	EE_DRY_MIX_1
	addwf	MTEMP4,w
	movwf	MTEMP1
	movlw	DRY_MIX_1
	addwf	MTEMP4,w
	movwf	FSR
	movfw	INDF
	movwf	MTEMP2
	call	WriteByteToEEPROM_Main
	
	; Clear bit of PARAMS_CHANGED3
	movfw	MTEMP3
	xorwf	PARAMS_CHANGED3
	
	goto	ChangedParamWritten3


;-----------------------------------------------------------------------------------------


CheckEncState	macro	WAS,bWAS, ALT,bALT, NOW,bNOW, COUNT, CCW, CW, Next
	btfsc	WAS,bWAS
	goto	$+D'12'
	
	; State was 0
	btfss	NOW,bNOW
	goto	Next				; No change, so skip to next
	; State changed: was 0, now 1
	incf	COUNT
	btfss	COUNT,2				; Check if state has been sampled changed 4 consecutive times
	goto	Next				; If not, then skip to next
	
	bsf	WAS,bWAS			; Record new state = 1
	btfsc	ALT,bALT			; Determine rotation direction
	call	CCW
	btfss	ALT,bALT
	call	CW
	goto	Next
	
	; State was 1
	btfsc	NOW,bNOW
	goto	Next				; No change, so skip to next
	; State changed: was 1, now 0
	incf	COUNT
	btfss	COUNT,2				; Check if state has been sampled changed 4 consecutive times
	goto	Next				; If not, then skip to next
	
	bcf	WAS,bWAS			; Record new state = 0
	btfsc	ALT,bALT			; Determine rotation direction
	call	CW
	btfss	ALT,bALT
	call	CCW
	goto	Next
endm


;-----------------------------------------------------------------------------------------


ReadControls
	
	; Check if any encoder states have changed
CheckDelayTime1A
	CheckEncState	ENC_STATES,DELAY_TIME_1_A, ENC_STATES,DELAY_TIME_1_B, PORT_DELAY_TIME_1_A, DB_DELAY_TIME_1, DelayTime1_Down, DelayTime1_Up, CheckDelayTime1B
CheckDelayTime1B
	CheckEncState	ENC_STATES,DELAY_TIME_1_B, ENC_STATES,DELAY_TIME_1_A, PORT_DELAY_TIME_1_B, DB_DELAY_TIME_1, DelayTime1_Up, DelayTime1_Down, CheckDelayTime2A
	
CheckDelayTime2A
	CheckEncState	ENC_STATES,DELAY_TIME_2_A, ENC_STATES,DELAY_TIME_2_B, PORT_DELAY_TIME_2_A, DB_DELAY_TIME_2, DelayTime2_Down, DelayTime2_Up, CheckDelayTime2B
CheckDelayTime2B
	CheckEncState	ENC_STATES,DELAY_TIME_2_B, ENC_STATES,DELAY_TIME_2_A, PORT_DELAY_TIME_2_B, DB_DELAY_TIME_2, DelayTime2_Up, DelayTime2_Down, CheckDelayTime3A
	
CheckDelayTime3A
	CheckEncState	ENC_STATES,DELAY_TIME_3_A, ENC_STATES,DELAY_TIME_3_B, PORT_DELAY_TIME_3_A, DB_DELAY_TIME_3, DelayTime3_Down, DelayTime3_Up, CheckDelayTime3B
CheckDelayTime3B
	CheckEncState	ENC_STATES,DELAY_TIME_3_B, ENC_STATES,DELAY_TIME_3_A, PORT_DELAY_TIME_3_B, DB_DELAY_TIME_3, DelayTime3_Up, DelayTime3_Down, CheckDelayTime4A
	
CheckDelayTime4A
	CheckEncState	ENC_STATES,DELAY_TIME_4_A, ENC_STATES,DELAY_TIME_4_B, PORT_DELAY_TIME_4_A, DB_DELAY_TIME_4, DelayTime4_Down, DelayTime4_Up, CheckDelayTime4B
CheckDelayTime4B
	CheckEncState	ENC_STATES,DELAY_TIME_4_B, ENC_STATES,DELAY_TIME_4_A, PORT_DELAY_TIME_4_B, DB_DELAY_TIME_4, DelayTime4_Up, DelayTime4_Down, CheckLFORateA

CheckLFORateA
	CheckEncState	ENC_STATES2,LFO_RATE_A, ENC_STATES2,LFO_RATE_B, PORT_LFO_RATE_A, DB_LFO_RATE, LFORate_Down, LFORate_Up, CheckLFORateB
CheckLFORateB
	CheckEncState	ENC_STATES2,LFO_RATE_B, ENC_STATES2,LFO_RATE_A, PORT_LFO_RATE_B, DB_LFO_RATE, LFORate_Up, LFORate_Down, CheckLFODepthA
	
CheckLFODepthA
	CheckEncState	ENC_STATES2,LFO_DEPTH_A, ENC_STATES2,LFO_DEPTH_B, PORT_LFO_DEPTH_A, DB_LFO_DEPTH, LFODepth_Down, LFODepth_Up, CheckLFODepthB
CheckLFODepthB
	CheckEncState	ENC_STATES2,LFO_DEPTH_B, ENC_STATES2,LFO_DEPTH_A, PORT_LFO_DEPTH_B, DB_LFO_DEPTH, LFODepth_Up, LFODepth_Down, CheckLFOPhaseA

CheckLFOPhaseA
	CheckEncState	ENC_STATES2,LFO_PHASE_A, ENC_STATES2,LFO_PHASE_B, PORT_LFO_PHASE_A, DB_LFO_PHASE, LFOPhase_Down, LFOPhase_Up, CheckLFOPhaseB
CheckLFOPhaseB
	CheckEncState	ENC_STATES2,LFO_PHASE_B, ENC_STATES2,LFO_PHASE_A, PORT_LFO_PHASE_B, DB_LFO_PHASE, LFOPhase_Up, LFOPhase_Down, CheckLFOWave
	
CheckLFOWave
	btfsc	ENC_STATES2,LFO_WAVE_A
	goto	CheckLFOWave_1
	
	; State was 0
	btfss	PORT_LFO_WAVE
	goto	CheckEnd			; No change, so end
	; State changed: was 0, now 1
	incf	DB_LFO_WAVE
	btfss	DB_LFO_WAVE,7			; Check if state has been sampled changed 128 consecutive times
	goto	CheckEnd			; If not, then end
	
	bsf	ENC_STATES2,LFO_WAVE_A		; Record new state = 1
	call	SwitchLFOWave			; Call SwitchLFOWave
	goto	CheckEnd
	
CheckLFOWave_1
	; State was 1
	btfsc	PORT_LFO_WAVE
	goto	CheckEnd			; No change, so end
	; State changed: was 1, now 0
	incf	DB_LFO_WAVE
	btfss	DB_LFO_WAVE,7			; Check if state has been sampled changed 128 consecutive times
	goto	CheckEnd			; If not, then end
	
	bcf	ENC_STATES2,LFO_WAVE_A		; Record new state = 0
	bcf	FLAGS,DONT_CHANGE_WAVE		; Reset flag
;	goto	CheckEnd
	
	
CheckEnd
	bcf	FLAGS,READ_CONTROLS	; Reset flag
	return
	
	
;-----------------------------------------------------------------------------------------
	
	
ReadChannel

CheckChannelA
	CheckEncState	ENC_STATES2,LFO_DEPTH_A, ENC_STATES2,LFO_DEPTH_B, PORT_LFO_DEPTH_A, DB_LFO_DEPTH, Channel_Down, Channel_Up, CheckChannelB
CheckChannelB
	CheckEncState	ENC_STATES2,LFO_DEPTH_B, ENC_STATES2,LFO_DEPTH_A, PORT_LFO_DEPTH_B, DB_LFO_DEPTH, Channel_Up, Channel_Down, CheckChannelEnd
	
CheckChannelEnd
	bcf	FLAGS,READ_CONTROLS	; Reset flag
	return

	
Channel_Up
	movfw	MIDI_CHANNEL		; If already = 15, return
	sublw	D'15'
	rz
	
	incf	MIDI_CHANNEL		; MIDI_CHANNEL ++
	bsf	PARAMS_CHANGED,bMIDI_CHANNEL
	return
	
Channel_Down
	movfw	MIDI_CHANNEL		; If already = 0, return
	rz
	
	decf	MIDI_CHANNEL		; MIDI_CHANNEL --
	bsf	PARAMS_CHANGED,bMIDI_CHANNEL
	return


;-----------------------------------------------------------------------------------------


DelayTime1_Up
	btfss	PORT_LFO_WAVE
	goto	Feedback13_Up

	incf	DELAY_TIME_1,w
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_DELAY_TIME_1
	bcf	PCLATH, 4		; Program bank 0
	return
	
DelayTime1_Down
	btfss	PORT_LFO_WAVE
	goto	Feedback13_Down

	movfw	DELAY_TIME_1
	sz
	decf	DELAY_TIME_1,w
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_DELAY_TIME_1
	bcf	PCLATH, 4		; Program bank 0
	return
	
	
DelayTime2_Up
	btfss	PORT_LFO_WAVE
	goto	Feedback24_Up

	incf	DELAY_TIME_2,w
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_DELAY_TIME_2
	bcf	PCLATH, 4		; Program bank 0
	return
	
DelayTime2_Down
	btfss	PORT_LFO_WAVE
	goto	Feedback24_Down

	movfw	DELAY_TIME_2
	sz
	decf	DELAY_TIME_2,w
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_DELAY_TIME_2
	bcf	PCLATH, 4		; Program bank 0
	return
	
	
DelayTime3_Up
	btfss	PORT_LFO_WAVE
	goto	DryMix13_Up

	incf	DELAY_TIME_3,w
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_DELAY_TIME_3
	bcf	PCLATH, 4		; Program bank 0
	return
	
DelayTime3_Down
	btfss	PORT_LFO_WAVE
	goto	DryMix13_Down

	movfw	DELAY_TIME_3
	sz
	decf	DELAY_TIME_3,w
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_DELAY_TIME_3
	bcf	PCLATH, 4		; Program bank 0
	return
	
	
DelayTime4_Up
	btfss	PORT_LFO_WAVE
	goto	DryMix24_Up

	incf	DELAY_TIME_4,w
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_DELAY_TIME_4
	bcf	PCLATH, 4		; Program bank 0
	return
	
DelayTime4_Down
	btfss	PORT_LFO_WAVE
	goto	DryMix24_Down

	movfw	DELAY_TIME_4
	sz
	decf	DELAY_TIME_4,w
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_DELAY_TIME_4
	bcf	PCLATH, 4		; Program bank 0
	return
	
	
LFORate_Up
	btfss	PORT_LFO_SELECT		; Check which LFO selected
	goto	LFO2Rate_Up
	
	; LFO1
	btfss	PORT_LFO_WAVE
	goto	LFO1Sync_Up

	incf	LFO1_RATE,w
	snz
	movfw	LFO1_RATE
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_LFO1_RATE
	bcf	PCLATH, 4		; Program bank 0
	return
	
LFO2Rate_Up
	; LFO2
	btfss	PORT_LFO_WAVE
	goto	LFO2Sync_Up

	incf	LFO2_RATE,w
	snz
	movfw	LFO2_RATE
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_LFO2_RATE
	bcf	PCLATH, 4		; Program bank 0
	return
	
LFORate_Down
	btfss	PORT_LFO_SELECT		; Check which LFO selected
	goto	LFO2Rate_Down
	
	; LFO1
	btfss	PORT_LFO_WAVE
	goto	LFO1Sync_Down

	movfw	LFO1_RATE
	sz
	decf	LFO1_RATE,w
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_LFO1_RATE
	bcf	PCLATH, 4		; Program bank 0
	return
	
LFO2Rate_Down
	; LFO2
	btfss	PORT_LFO_WAVE
	goto	LFO2Sync_Down

	movfw	LFO2_RATE
	sz
	decf	LFO2_RATE,w
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_LFO2_RATE
	bcf	PCLATH, 4		; Program bank 0
	return
	
	
LFODepth_Up
	btfss	PORT_LFO_SELECT		; Check which LFO selected
	goto	LFO2Depth_Up
	
	; LFO1
	btfss	PORT_LFO_WAVE
	goto	DoubleDelayMode12_Up
	
	incf	LFO1_DEPTH,w
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_LFO1_DEPTH
	bcf	PCLATH, 4		; Program bank 0
	return
	
LFO2Depth_Up
	; LFO2
	btfss	PORT_LFO_WAVE
	goto	DoubleDelayMode34_Up
	
	incf	LFO2_DEPTH,w
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_LFO2_DEPTH
	bcf	PCLATH, 4		; Program bank 0
	return
	
LFODepth_Down
	btfss	PORT_LFO_SELECT		; Check which LFO selected
	goto	LFO2Depth_Down
	
	; LFO1
	btfss	PORT_LFO_WAVE
	goto	DoubleDelayMode12_Down
	
	movfw	LFO1_DEPTH
	sz
	decf	LFO1_DEPTH,w
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_LFO1_DEPTH
	bcf	PCLATH, 4		; Program bank 0
	return
	
LFO2Depth_Down
	; LFO2
	btfss	PORT_LFO_WAVE
	goto	DoubleDelayMode34_Down
	
	movfw	LFO2_DEPTH
	sz
	decf	LFO2_DEPTH,w
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_LFO2_DEPTH
	bcf	PCLATH, 4		; Program bank 0
	return
	
	
LFOPhase_Up
	btfss	PORT_LFO_SELECT		; Check which LFO selected
	goto	LFO2Phase_Up
	
	; LFO1
	btfss	PORT_LFO_WAVE
	goto	ResamplingMode12_On

	incf	LFO1_PHASE,w
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_LFO1_PHASE
	bcf	PCLATH, 4		; Program bank 0
	return
	
LFO2Phase_Up
	; LFO2
	btfss	PORT_LFO_WAVE
	goto	ResamplingMode34_On

	incf	LFO2_PHASE,w
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_LFO2_PHASE
	bcf	PCLATH, 4		; Program bank 0
	return
	
LFOPhase_Down
	btfss	PORT_LFO_SELECT		; Check which LFO selected
	goto	LFO2Phase_Down
	
	; LFO1
	btfss	PORT_LFO_WAVE
	goto	ResamplingMode12_Off

	movfw	LFO1_PHASE
	sz
	decf	LFO1_PHASE,w
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_LFO1_PHASE
	bcf	PCLATH, 4		; Program bank 0
	return
	
LFO2Phase_Down
	; LFO2
	btfss	PORT_LFO_WAVE
	goto	ResamplingMode34_Off

	movfw	LFO2_PHASE
	sz
	decf	LFO2_PHASE,w
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_LFO2_PHASE
	bcf	PCLATH, 4		; Program bank 0
	return
	
	
SwitchLFOWave
	btfsc	FLAGS,DONT_CHANGE_WAVE	; If other control has been changed while button held, do nothing
	return

	btfss	PORT_LFO_SELECT		; Check which LFO selected
	goto	SwitchLFO2Wave
	
	; LFO1
	incf	LFO1_WAVE,w
	movwf	MTEMP4
	sublw	5
	snz
	clrf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Change_LFO1_WAVE
	bcf	PCLATH, 4		; Program bank 0
	return
	
SwitchLFO2Wave
	; LFO2
	incf	LFO2_WAVE,w
	movwf	MTEMP4
	sublw	5
	snz
	clrf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Change_LFO2_WAVE
	bcf	PCLATH, 4		; Program bank 0
	return

LFO1Sync_Up
	bsf	FLAGS,DONT_CHANGE_WAVE	; Set flag

	incf	LFO1_SYNC,w
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_LFO1_SYNC
	bcf	PCLATH, 4		; Program bank 0
	return

LFO1Sync_Down
	bsf	FLAGS,DONT_CHANGE_WAVE	; Set flag

	movfw	LFO1_SYNC
	sz
	decf	LFO1_SYNC,w
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_LFO1_SYNC
	bcf	PCLATH, 4		; Program bank 0
	return

LFO2Sync_Up
	bsf	FLAGS,DONT_CHANGE_WAVE	; Set flag

	incf	LFO2_SYNC,w
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_LFO2_SYNC
	bcf	PCLATH, 4		; Program bank 0
	return

LFO2Sync_Down
	bsf	FLAGS,DONT_CHANGE_WAVE	; Set flag

	movfw	LFO2_SYNC
	sz
	decf	LFO2_SYNC,w
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_LFO2_SYNC
	bcf	PCLATH, 4		; Program bank 0
	return
	
ResamplingMode12_Off
	bsf	FLAGS,DONT_CHANGE_WAVE	; Set flag

	clrf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_RESAMPLING_MODE_12
	bcf	PCLATH, 4		; Program bank 0
	return
	
ResamplingMode12_On
	bsf	FLAGS,DONT_CHANGE_WAVE	; Set flag

	clrf	MTEMP4
	decf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_RESAMPLING_MODE_12
	bcf	PCLATH, 4		; Program bank 0
	return
	
ResamplingMode34_Off
	bsf	FLAGS,DONT_CHANGE_WAVE	; Set flag

	clrf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_RESAMPLING_MODE_34
	bcf	PCLATH, 4		; Program bank 0
	return
	
ResamplingMode34_On
	bsf	FLAGS,DONT_CHANGE_WAVE	; Set flag

	clrf	MTEMP4
	decf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_RESAMPLING_MODE_34
	bcf	PCLATH, 4		; Program bank 0
	return

DoubleDelayMode12_Up
	bsf	FLAGS,DONT_CHANGE_WAVE	; Set flag
	
	clrf	MTEMP4
	btfsc	RESAMPLING_MODE, HALF_SAMPLING_RATE_12
	bsf	MTEMP4, 0
	btfsc	RESAMPLING_MODE, SINGLE_INPUT_12
	bsf	MTEMP4, 1
	
	incf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_DoubleDelayMode12
	bcf	PCLATH, 4		; Program bank 0
	return

DoubleDelayMode12_Down
	bsf	FLAGS,DONT_CHANGE_WAVE	; Set flag
	
	clrf	MTEMP4
	btfsc	RESAMPLING_MODE, HALF_SAMPLING_RATE_12
	bsf	MTEMP4, 0
	btfsc	RESAMPLING_MODE, SINGLE_INPUT_12
	bsf	MTEMP4, 1
	
	movfw	MTEMP4
	sz
	decf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_DoubleDelayMode12
	bcf	PCLATH, 4		; Program bank 0
	return

DoubleDelayMode34_Up
	bsf	FLAGS,DONT_CHANGE_WAVE	; Set flag
	
	clrf	MTEMP4
	btfsc	RESAMPLING_MODE, HALF_SAMPLING_RATE_34
	bsf	MTEMP4, 0
	btfsc	RESAMPLING_MODE, SINGLE_INPUT_34
	bsf	MTEMP4, 1
	
	incf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_DoubleDelayMode34
	bcf	PCLATH, 4		; Program bank 0
	return

DoubleDelayMode34_Down
	bsf	FLAGS,DONT_CHANGE_WAVE	; Set flag
	
	clrf	MTEMP4
	btfsc	RESAMPLING_MODE, HALF_SAMPLING_RATE_34
	bsf	MTEMP4, 0
	btfsc	RESAMPLING_MODE, SINGLE_INPUT_34
	bsf	MTEMP4, 1
	
	movfw	MTEMP4
	sz
	decf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_DoubleDelayMode34
	bcf	PCLATH, 4		; Program bank 0
	return

DryMix13_Up
	bsf	FLAGS,DONT_CHANGE_WAVE	; Set flag

	btfss	PORT_LFO_SELECT		; Check which delay referred to
	goto	DryMix3_Up
	
DryMix1_Up
	incf	DRY_MIX_1,w
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_DRY_MIX_1
	bcf	PCLATH, 4		; Program bank 0
	return

DryMix3_Up
	incf	DRY_MIX_3,w
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_DRY_MIX_3
	bcf	PCLATH, 4		; Program bank 0
	return

DryMix13_Down
	bsf	FLAGS,DONT_CHANGE_WAVE	; Set flag

	btfss	PORT_LFO_SELECT		; Check which delay referred to
	goto	DryMix3_Down
	
DryMix1_Down
	movfw	DRY_MIX_1
	sz
	decf	DRY_MIX_1,w
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_DRY_MIX_1
	bcf	PCLATH, 4		; Program bank 0
	return

DryMix3_Down
	movfw	DRY_MIX_3
	sz
	decf	DRY_MIX_3,w
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_DRY_MIX_3
	bcf	PCLATH, 4		; Program bank 0
	return

DryMix24_Up
	bsf	FLAGS,DONT_CHANGE_WAVE	; Set flag

	btfss	PORT_LFO_SELECT		; Check which delay referred to
	goto	DryMix4_Up
	
DryMix2_Up
	incf	DRY_MIX_2,w
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_DRY_MIX_2
	bcf	PCLATH, 4		; Program bank 0
	return

DryMix4_Up
	incf	DRY_MIX_4,w
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_DRY_MIX_4
	bcf	PCLATH, 4		; Program bank 0
	return

DryMix24_Down
	bsf	FLAGS,DONT_CHANGE_WAVE	; Set flag

	btfss	PORT_LFO_SELECT		; Check which delay referred to
	goto	DryMix4_Down
	
DryMix2_Down
	movfw	DRY_MIX_2
	sz
	decf	DRY_MIX_2,w
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_DRY_MIX_2
	bcf	PCLATH, 4		; Program bank 0
	return

DryMix4_Down
	movfw	DRY_MIX_4
	sz
	decf	DRY_MIX_4,w
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_DRY_MIX_4
	bcf	PCLATH, 4		; Program bank 0
	return

Feedback13_Up
	bsf	FLAGS,DONT_CHANGE_WAVE	; Set flag

	btfss	PORT_LFO_SELECT		; Check which delay referred to
	goto	Feedback3_Up
	
Feedback1_Up
	incf	FEEDBACK_1,w
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_FEEDBACK_1
	bcf	PCLATH, 4		; Program bank 0
	return

Feedback3_Up
	incf	FEEDBACK_3,w
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_FEEDBACK_3
	bcf	PCLATH, 4		; Program bank 0
	return

Feedback13_Down
	bsf	FLAGS,DONT_CHANGE_WAVE	; Set flag

	btfss	PORT_LFO_SELECT		; Check which delay referred to
	goto	Feedback3_Down
	
Feedback1_Down
	movfw	FEEDBACK_1
	sz
	decf	FEEDBACK_1,w
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_FEEDBACK_1
	bcf	PCLATH, 4		; Program bank 0
	return

Feedback3_Down
	movfw	FEEDBACK_3
	sz
	decf	FEEDBACK_3,w
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_FEEDBACK_3
	bcf	PCLATH, 4		; Program bank 0
	return

Feedback24_Up
	bsf	FLAGS,DONT_CHANGE_WAVE	; Set flag

	btfss	PORT_LFO_SELECT		; Check which delay referred to
	goto	Feedback4_Up
	
Feedback2_Up
	incf	FEEDBACK_2,w
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_FEEDBACK_2
	bcf	PCLATH, 4		; Program bank 0
	return

Feedback4_Up
	incf	FEEDBACK_4,w
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_FEEDBACK_4
	bcf	PCLATH, 4		; Program bank 0
	return

Feedback24_Down
	bsf	FLAGS,DONT_CHANGE_WAVE	; Set flag

	btfss	PORT_LFO_SELECT		; Check which delay referred to
	goto	Feedback4_Down
	
Feedback2_Down
	movfw	FEEDBACK_2
	sz
	decf	FEEDBACK_2,w
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_FEEDBACK_2
	bcf	PCLATH, 4		; Program bank 0
	return

Feedback4_Down
	movfw	FEEDBACK_4
	sz
	decf	FEEDBACK_4,w
	movwf	MTEMP4
	bsf	MIDI_FLAGS,SEND_MIDI_CC
	bsf	PCLATH, 4		; Program bank 2
	call	Check_FEEDBACK_4
	bcf	PCLATH, 4		; Program bank 0
	return


;-----------------------------------------------------------------------------------------

	
; Call during main loop (or initialisation) to read byte from EEPROM
; 
; On call:
; --------
; RAM bank 0 selected
; MTEMP1 contains address location to read from
; 
; On return:
; ----------
; RAM bank 0 selected
; w contains byte read
;ReadByteFromEEPROM_Main
;	bcf	FLAGS,INTERRUPTS_OFF
;	btfss	INTCON,GIE
;	bsf	FLAGS,INTERRUPTS_OFF
;
;	bsf	STATUS,RP1		; Bank 2
;	bcf	INTCON,GIE		; Disable interrupts
;	movfw	MTEMP1			; Address to read
;	movwf	EEADR
;	bsf	STATUS,RP0		; Bank 3
;	bsf	EECON1,RD		; Start read operation
;	bcf	STATUS,RP0		; Bank 2
;	movfw	EEDATA			; Read byte
;	bcf	STATUS,RP1		; Bank 0
;	btfss	FLAGS,INTERRUPTS_OFF
;	bsf	INTCON,GIE		; Re-enable interrupts
;	return
	

; Call during interrupt (or initialisation) to read byte from EEPROM
; 
; On call:
; --------
; RAM bank 0 selected
; TEMP1 contains address location to read from
; 
; On return:
; ----------
; RAM bank 0 selected
; w contains byte read
ReadByteFromEEPROM_Int
	bsf	STATUS,RP1		; Bank 2
	movfw	TEMP1			; Address to read
	movwf	EEADR
	bsf	STATUS,RP0		; Bank 3
	bsf	EECON1,RD		; Start read operation
	bcf	STATUS,RP0		; Bank 2
	movfw	EEDATA			; Read byte
	bcf	STATUS,RP1		; Bank 0
	return
	

; Call during main loop (or initialisation) to write byte to EEPROM
; 
; On call:
; --------
; RAM bank 0 selected
; MTEMP1 contains address location to write to
; MTEMP2 contains byte to write
; 
; On return:
; ----------
; RAM bank 0 selected
WriteByteToEEPROM_Main
	bcf	FLAGS,INTERRUPTS_OFF
	btfss	INTCON,GIE
	bsf	FLAGS,INTERRUPTS_OFF
	
	bsf	STATUS,RP1		; Bank 2
	
	bcf	INTCON,GIE		; Disable interrupts
	
	movfw	MTEMP1			; Address to write
	movwf	EEADR
	movfw	MTEMP2			; Data to write
	movwf	EEDATA
	
	bsf	STATUS,RP0		; Bank 3
	
	bsf	EECON1,WREN		; Enable writes
	
	movlw	0x55			; Special write sequence
	movwf	EECON2
	movlw	0xAA
	movwf	EECON2
	bsf	EECON1,WR
	
	bcf	EECON1,WREN		; Disable writes
	
	btfsc	EECON1,WR		; Wait for write to complete
	goto	$-1
	
	bcf	STATUS,RP0		; Bank 0
	bcf	STATUS,RP1
	
	btfss	FLAGS,INTERRUPTS_OFF
	bsf	INTCON,GIE		; Re-enable interrupts
	return
	
	

; Call during interrupt (or initialisation) to write byte to EEPROM
; 
; On call:
; --------
; RAM bank 0 selected
; TEMP1 contains address location to write to
; TEMP2 contains byte to write
; 
; On return:
; ----------
; RAM bank 0 selected
WriteByteToEEPROM_Int
	bsf	STATUS,RP1		; Bank 2
	
	movfw	TEMP1			; Address to write
	movwf	EEADR
	movfw	TEMP2			; Data to write
	movwf	EEDATA
	
	bsf	STATUS,RP0		; Bank 3
	
	bsf	EECON1,WREN		; Enable writes
	
	movlw	0x55			; Special write sequence
	movwf	EECON2
	movlw	0xAA
	movwf	EECON2
	bsf	EECON1,WR
	
	bcf	EECON1,WREN		; Disable writes
	
	btfsc	EECON1,WR		; Wait for write to complete
	goto	$-1
	
	bcf	STATUS,RP0		; Bank 0
	bcf	STATUS,RP1
	return
	
	
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------


	
;***********************************************************************
;***********************************************************************



;#######################################################################


	; Program bank 1
	
	; ISRs and Maths library are here
	
	org	0x0800
	

;***********************************************************************


; Called every 2.62144ms
; Updates 1 digit of LED per call
UpdateLEDs
	bcf	INTCON,TMR0IF	; Reset TMR0 flag
	
	incf16	EEPROM_DATA_WRITE_L	; Increment EEPROM data write check counter
	
	movfw	LED_INTERRUPT_COUNT	; Return if LED display is turned off
	rz
	
	incf	LED_INTERRUPT_COUNT_L
	btfss	LED_INTERRUPT_COUNT_L,3
	goto	UpdateLEDs_SkipIncrCount
	
	clrf	LED_INTERRUPT_COUNT_L
	incf	LED_INTERRUPT_COUNT	; Check if LED display needs to be turned off
	jz	UpdateLEDs_TurnOff
	
UpdateLEDs_SkipIncrCount
	; Check which digit to display
	btfsc	FLAGS,CURRENT_DIGIT_2
	goto	UpdateLEDs_Digit2
	
	btfsc	FLAGS,CURRENT_DIGIT_3
	goto	UpdateLEDs_Digit3
	
UpdateLEDs_Digit1
	bcf	PORTC,DIGIT3	; Turn off digit 3
	movfw	LED_DIGIT1	; Load digit 1 value into PORTD
	movwf	PORTD
	bsf	PORTC,DIGIT1	; Turn on digit 1
	
	bsf	FLAGS,CURRENT_DIGIT_2	; Set digit to update next time
	return
	
UpdateLEDs_Digit2
	bcf	PORTC,DIGIT1	; Turn off digit 1
	movfw	LED_DIGIT2	; Load digit 2 value into PORTD
	movwf	PORTD
	bsf	PORTC,DIGIT2	; Turn on digit 2
	
	bcf	FLAGS,CURRENT_DIGIT_2	; Set digit to update next time
	bsf	FLAGS,CURRENT_DIGIT_3
	return
	
UpdateLEDs_Digit3
	bcf	PORTC,DIGIT2	; Turn off digit 2
	movfw	LED_DIGIT3	; Load digit 3 value into PORTD
	movwf	PORTD
	bsf	PORTC,DIGIT3	; Turn on digit 3
	
	bcf	FLAGS,CURRENT_DIGIT_3	; Set digit to update next time
	return
	
UpdateLEDs_TurnOff
	movlw	b'11111000'	; Turn off all digits
	andwf	PORTC
	
	return
	
	
;***********************************************************************

	
Timer1Int
	movlw	0xFD		; Reset timer for 0.25ms between each sampling
	movwf	TMR1H
	movlw	0x09
	movwf	TMR1L
	
	bcf	PIR1,TMR1IF	; Clear interrupt flag
	
	bsf	FLAGS,READ_CONTROLS	; Cause controls to be read in main loop
	return

	
;***********************************************************************


Timer2Int
	bcf	PIR1,TMR2IF	; Clear interrupt flag
	incfsz	TIMER2_H	; Increment high byte
	return
	decf	TIMER2_H	; Ensure high byte never wraps around for very slow tempos
	return
	
	
;***********************************************************************

	
	
PointToNextIncomingBufferByte	macro
	incf	FSR			; Increment pointer
	btfsc	FSR,4			; Check not wrapped
	goto	$+3
	movlw	low MIDI_MSG_BUFFER	; Wrap pointer
	movwf	FSR
	
	movfw	FSR			; Check for ring buffer overrun
	subwf	MIDI_MSG_PROCESS_PTR,w
	jz	MIDIByteReceived_RingBuffOver
endm

MIDIByteReceived_RingBuffOver
	clrf	MIDI_MSG1		; Clear status byte and flags
	movlw	b'11111100'
	andwf	MIDI_FLAGS
	return


; RAM bank 0
MIDIByteReceived

	movfw	RCSTA			; Read and save and error code
	movwf	TEMP2
	
	movfw	RCREG			; Read MIDI byte
	movwf	TEMP1			; Save in TEMP1
	
	btfsc	TEMP2,OERR		; Check for overrun error
	goto	MIDIByteReceived_Error
	btfsc	TEMP2,FERR		; Check for framing error
	goto	MIDIByteReceived_Error
	
	
	btfss	TEMP1,7			; Header or data byte?
	goto	MIDIByteReceived_DataByte
	
; header byte
	movlw	b'11110000'		; System message
	andwf	TEMP1,w
	sublw	b'11110000'
	jz	MIDIByteReceived_System
	
	; Voice message
	movlw	b'11111100'		; Clear flags
	andwf	MIDI_FLAGS
	
	movfw	TEMP1			; Store status byte
	movwf	MIDI_MSG1
	
	movlw	b'11100000'		; 2 byte messages (Program Change + Aftertouch)
	andwf	TEMP1,w
	sublw	b'11000000'
	sz				; If not 2 byte message, set flag
	bsf	MIDI_FLAGS,MSG_LEN
	
	return
	
MIDIByteReceived_System
	btfss	TEMP1,3			; System common message?
	goto	MIDIByteReceived_SystemCommon
	
	; System realtime message
	movfw	MIDI_MSG_INCOMING_PTR
	movwf	FSR
	PointToNextIncomingBufferByte	; Point INDF to next byte of ring buffer
	movfw	FSR
	movwf	MIDI_MSG_INCOMING_PTR
	
	movfw	TEMP1			; Write byte
	movwf	INDF
	
	return
	
MIDIByteReceived_SystemCommon
	clrf	MIDI_MSG1		; Clear status buffer
	
	return				; Ignore system common messages
	

MIDIByteReceived_DataByte
	movfw	MIDI_MSG1		; Drop byte if status is 0
	rz
	
	btfsc	MIDI_FLAGS,MSG_LEN	; Test if expecting 3 byte message
	goto	MIDIByteReceived_DataByte_3byte
	
	; 2 byte message
	movfw	MIDI_MSG_INCOMING_PTR
	movwf	FSR
	PointToNextIncomingBufferByte	; Point INDF to next byte of ring buffer
	
	movfw	MIDI_MSG1		; Write status byte
	movwf	INDF
	
	PointToNextIncomingBufferByte	; Point INDF to next byte of ring buffer
	movfw	FSR
	movwf	MIDI_MSG_INCOMING_PTR
	
	movfw	TEMP1			; Write data byte
	movwf	INDF
	
	return
	
MIDIByteReceived_DataByte_3byte
	btfsc	MIDI_FLAGS,WHICH_BYTE	; Test if this is the 2nd data byte
	goto	MIDIByteReceived_DataByte_3byte2
	
	; 3 byte message, 1st data byte
	movfw	TEMP1			; Store data byte
	movwf	MIDI_MSG2
	
	bsf	MIDI_FLAGS,WHICH_BYTE	; Set flag to expect 2nd data byte
	
	return
	
MIDIByteReceived_DataByte_3byte2
	; 3 byte message, 2nd data byte
	movfw	MIDI_MSG_INCOMING_PTR
	movwf	FSR
	PointToNextIncomingBufferByte	; Point INDF to next byte of ring buffer
	
	movfw	MIDI_MSG1		; Write status byte
	movwf	INDF
	
	PointToNextIncomingBufferByte	; Point INDF to next byte of ring buffer
	
	movfw	MIDI_MSG2		; Write 1st data byte
	movwf	INDF
	
	PointToNextIncomingBufferByte	; Point INDF to next byte of ring buffer
	movfw	FSR
	movwf	MIDI_MSG_INCOMING_PTR
	
	movfw	TEMP1			; Write 2nd data byte
	movwf	INDF
	
	bcf	MIDI_FLAGS,WHICH_BYTE	; Set flag to expect 1st data byte
	
	return
	
	
MIDIByteReceived_Error
	bcf	RCSTA,CREN		; Reset receive logic / clear error flag
	bsf	RCSTA,CREN
	return
	
	
;***********************************************************************
;***********************************************************************
	
	
; Maths Functions
; #include <maths.inc>		; Not needed
	
	
;#######################################################################


	; Program bank 2
	
	org	0x1000
	
	
;-----------------------------------------------------------------------------------------
	
	
; Lookup tables for LED digits

; Have to take care table doesn't exceed size of single memory page
; TABLE_START	EQU	$
; 	org	$ + 0xFF - ($ % 0x100)

GetLEDNumber
	addwf	PCL
LEDText_Base
;            7  6  5  4  3  2  1  0
; Bit order: c  DP d  e  g  b  f  a

LEDNumbers
	dt	b'01001000'	; 0
	dt	b'01111011'	; 1
	dt	b'11000010'	; 2
	dt	b'01010010'	; 3
	dt	b'01110001'	; 4
	dt	b'01010100'	; 5
	dt	b'01000100'	; 6
	dt	b'01111000'	; 7
	dt	b'01000000'	; 8
	dt	b'01010000'	; 9

LEDText_Sin
	dt	b'01010100'	; S
	dt	b'11101111'	; i
	dt	b'01100111'	; n
;	dt	b'01001111'	; u
;	dt	b'11111000'	; ^n
;	dt	b'01001111'	; u

LEDText_Tri
	dt	b'11000101'	; t
	dt	b'11100111'	; r
	dt	b'11101111'	; i
;	dt	b'01111111'
;	dt	b'01111101'
;	dt	b'11111101'

LEDText_SawUp
	dt	b'01001111'	; u
	dt	b'11100000'	; P
	dt	b'11111111'	; ' '
;	dt	b'11011111'	; _
;	dt	b'11110111'	; -
;	dt	b'11111110'	; topscore

LEDText_SawDown
	dt	b'01000011'	; d
	dt	b'01100111'	; n
	dt	b'11111111'	; ' '
;	dt	b'11111110'	; topscore
;	dt	b'11110111'	; -
;	dt	b'11011111'	; _

LEDText_Square
	dt	b'01010100'	; S
	dt	b'01110000'	; q
	dt	b'11100111'	; r

LEDText_C
	dt	b'11001100'	; C

LEDText_Minus
	dt	b'11110111'	; -

LEDText_rof
	dt	b'11100111'	; r
	dt	b'01000111'	; o
	dt	b'11100100'	; f

LEDText_ron
	dt	b'11100111'	; r
	dt	b'01000111'	; o
	dt	b'01100111'	; n
	
LEDText_L
	dt	b'11001101'	; L
	
LEDText_f
	dt	b'11100100'	; f

LEDText_Off
	dt	b'01001000'	; O
	dt	b'11100100'	; f
	dt	b'11100100'	; f

LEDText_16_
	dt	b'01111011'	; 1
	dt	b'01000100'	; 6
	dt	b'11011111'	; _

LEDText_8_
	dt	b'01000000'	; 8
	dt	b'11011111'	; _
	dt	b'01111011'	; 1

LEDText_4_
	dt	b'01110001'	; 4
	dt	b'11011111'	; _
	dt	b'01111011'	; 1

LEDText_2_
	dt	b'11000010'	; 2
	dt	b'11011111'	; _
	dt	b'01111011'	; 1

LEDText_1_
	dt	b'01111011'	; 1
	dt	b'11011111'	; _
	dt	b'01111011'	; 1

LEDText_3_4
	dt	b'01010010'	; 3
	dt	b'11011111'	; _
	dt	b'01110001'	; 4

LEDText__2
	dt	b'01111011'	; 1
	dt	b'11011111'	; _
	dt	b'11000010'	; 2

LEDText__3
	dt	b'01111011'	; 1
	dt	b'11011111'	; _
	dt	b'01010010'	; 3

LEDText__4
	dt	b'01111011'	; 1
	dt	b'11011111'	; _
	dt	b'01110001'	; 4

LEDText__6
	dt	b'01111011'	; 1
	dt	b'11011111'	; _
	dt	b'01000100'	; 6

LEDText__8
	dt	b'01111011'	; 1
	dt	b'11011111'	; _
	dt	b'01000000'	; 8

LEDText__12
	dt	b'11011111'	; _
	dt	b'01111011'	; 1
	dt	b'11000010'	; 2

LEDText__16
	dt	b'11011111'	; _
	dt	b'01111011'	; 1
	dt	b'01000100'	; 6
	
LEDText_dt1
	dt	b'01000011'	; d
	dt	b'11000101'	; t
	dt	b'01111011'	; 1
	
LEDText_dt2
	dt	b'01000011'	; d
	dt	b'11000101'	; t
	dt	b'11000010'	; 2
	
LEDText_dt2dot
	dt	b'01000011'	; d
	dt	b'11000101'	; t
	dt	b'10000010'	; 2.
	

;-----------------------------------------------------------------------------------------


; Lookup table for LFO Sync - number of MIDI clocks per period

; Have to take care table doesn't cross page boundary

LookupLFOSync_H
	addwf	PCL
	retlw	0x00	; 0=Off
	retlw	0x06	; 1=16
	retlw	0x03	; 2=8
	retlw	0x01	; 3=4
	retlw	0x00	; 4=2
	retlw	0x00	; 5=1
	retlw	0x00	; 6=3/4
	retlw	0x00	; 7=1/2
	retlw	0x00	; 8=1/3
	retlw	0x00	; 9=1/4
	retlw	0x00	; 10=1/6
	retlw	0x00	; 11=1/8
	retlw	0x00	; 12=1/12
	retlw	0x00	; 13=1/16
	
LookupLFOSync_L
	addwf	PCL
	retlw	0x00	; 0=Off
	retlw	0x00	; 1=16
	retlw	0x00	; 2=8
	retlw	0x80	; 3=4
	retlw	0xC0	; 4=2
	retlw	0x60	; 5=1
	retlw	0x48	; 6=3/4
	retlw	0x30	; 7=1/2
	retlw	0x20	; 8=1/3
	retlw	0x18	; 9=1/4
	retlw	0x10	; 10=1/6
	retlw	0x0C	; 11=1/8
	retlw	0x08	; 12=1/12
	retlw	0x06	; 13=1/16


;-----------------------------------------------------------------------------------------


; RAM Bank 0
PointToNextProcessBufferByte
	incf	MIDI_MSG_PROCESS_PTR	; Increment pointer
	btfsc	MIDI_MSG_PROCESS_PTR,4	; Check not wrapped
	return
	movlw	low MIDI_MSG_BUFFER	; Wrap pointer
	movwf	MIDI_MSG_PROCESS_PTR
	return
	

;-----------------------------------------------------------------------------------------

	; Implemented messages:
	;
	; 1011cccc cccccccc vvvvvvvv  Control Change
	
; Parse MIDI message here
; RAM Bank 0
ProcessMIDI
	call	PointToNextProcessBufferByte
	
	movlw	high ProcessMIDI_NoteOff	; Ready PCLATH for jump table
	movwf	PCLATH
	
	bcf	STATUS,IRP		; Indirectly address banks 0-1
	movfw	MIDI_MSG_PROCESS_PTR
	movwf	FSR
	
	movfw	INDF			; Get and save MIDI channel
	andlw	b'00001111'
	movwf	MTEMP1
	
	rrf	INDF,w			; Get message
	movwf	MTEMP2
	rrf	MTEMP2
	rrf	MTEMP2
	rrf	MTEMP2,w
	andlw	b'00000111'
	
	addwf	PCL			; Jump to handler  - *** HAVE TO TAKE CARE NOT TO CROSS PAGE BOUNDARY! ***
	goto	ProcessMIDI_NoteOff
	goto	ProcessMIDI_NoteOn
	goto	ProcessMIDI_PolyAftertouch
	goto	ProcessMIDI_ControlChange
	goto	ProcessMIDI_ProgramChange
	goto	ProcessMIDI_Aftertouch
	goto	ProcessMIDI_PitchBend
	goto	ProcessMIDI_System


;-----------------------------------------------------------------------------------------
	
; RAM Bank 0
ProcessMIDI_NoteOff
	call	PointToNextProcessBufferByte
	call	PointToNextProcessBufferByte
	return
	
	
;-----------------------------------------------------------------------------------------
	
; RAM Bank 0
ProcessMIDI_NoteOn
	call	PointToNextProcessBufferByte
	call	PointToNextProcessBufferByte
	return
	
	
;-----------------------------------------------------------------------------------------
	
; RAM Bank 0
ProcessMIDI_PolyAftertouch
	call	PointToNextProcessBufferByte
	call	PointToNextProcessBufferByte
	return
	
	
;-----------------------------------------------------------------------------------------
	
; RAM Bank 0
ProcessMIDI_ProgramChange
	call	PointToNextProcessBufferByte
	return
	
	
;-----------------------------------------------------------------------------------------
	
; RAM Bank 0
ProcessMIDI_Aftertouch
	call	PointToNextProcessBufferByte
	return
	
	
;-----------------------------------------------------------------------------------------
	
; RAM Bank 0
ProcessMIDI_PitchBend
	call	PointToNextProcessBufferByte
	call	PointToNextProcessBufferByte
	return
	
	
;-----------------------------------------------------------------------------------------
	
; RAM Bank 0
ProcessMIDI_System
	; Check system message value
	movfw	MTEMP1		
	sublw	0x0A		; MIDI Start ?
	jz	ProcessMIDI_System_Start
	movfw	MTEMP1
	sublw	0x08		; MIDI Clock ?
	jz	ProcessMIDI_System_Clock
	return
	
ProcessMIDI_System_Start
	clrf	TMR2		; Reset timer 2
	clrf	TIMER2_H
	clrf	MIDI_CLOCK_COUNT1_L ; Reset MIDI clock counters
	clrf	MIDI_CLOCK_COUNT1_H
	clrf	MIDI_CLOCK_COUNT2_L
	clrf	MIDI_CLOCK_COUNT2_H
	
	; SPI_MSG - Chip select & Message type nybble: c000mmmm
	; AARGB1  - Value to transmit byte
	movlw	b'00000110'
	movwf	SPI_MSG
	
	movfw	LFO1_SYNC	; Transmit LFO1 Sync Start
	movwf	AARGB1
	call	TransmitSPI
	
	bsf	SPI_MSG,7	; Transmit LFO2 Sync Start
	movfw	LFO2_SYNC
	movwf	AARGB1
	goto	TransmitSPI
	

ProcessMIDI_System_Clock
	incf	MIDI_CLOCK_COUNT1_L
	snz
	incf	MIDI_CLOCK_COUNT1_H
	incf	MIDI_CLOCK_COUNT2_L
	snz
	incf	MIDI_CLOCK_COUNT2_H
	btfsc	MIDI_CLOCK_COUNT1_L,0
	return
	
	; After 2 clocks, send 1/48 bar length to DSP PIC
	movfw	TMR2		; Save and reset timer 2
	clrf	TMR2
	movwf	AARGB1
	
	movfw	TIMER2_H	; Save and reset timer 2 high byte
	clrf	TIMER2_H
	movwf	AARGB2
	
	; SPI_MSG - Chip select & Message type nybble: c000mmmm
	; AARGB1  - Value to transmit byte
	movlw	b'00001100'
	movwf	SPI_MSG
	
	call	TransmitSPI	; Transmit LFO1 Sync Message
	bsf	SPI_MSG,7	; Transmit LFO2 Sync Message
	call	TransmitSPI
	
	; SPI_MSG - Chip select & Message type nybble: c000mmmm
	; AARGB1  - Value to transmit byte
	movlw	b'00001101'
	movwf	SPI_MSG
	movfw	AARGB2
	movwf	AARGB1

	call	TransmitSPI	; Transmit LFO1 Sync Message
	bsf	SPI_MSG,7	; Transmit LFO2 Sync Message
	call	TransmitSPI
	
Check_LFO1_SyncStart
	; Check if need to send LFO1 Sync Start
	movfw	MIDI_CLOCK_PERIOD1_L
	subwf	MIDI_CLOCK_COUNT1_L,w
	jnz	Check_LFO2_SyncStart
	movfw	MIDI_CLOCK_PERIOD1_H
	subwf	MIDI_CLOCK_COUNT1_H,w
	jnz	Check_LFO2_SyncStart
	
	; SPI_MSG - Chip select & Message type nybble: c000mmmm
	; AARGB1  - Value to transmit byte
	movlw	b'00000110'
	movwf	SPI_MSG
	movfw	LFO1_SYNC	; Transmit LFO1 Sync Start
	movwf	AARGB1
	call	TransmitSPI
	
	clrf	MIDI_CLOCK_COUNT1_L ; Reset MIDI clock count
	clrf	MIDI_CLOCK_COUNT1_H
	
Check_LFO2_SyncStart
	; Check if need to send LFO1 Sync Start
	movfw	MIDI_CLOCK_PERIOD2_L
	subwf	MIDI_CLOCK_COUNT2_L,w
	rnz
	movfw	MIDI_CLOCK_PERIOD2_H
	subwf	MIDI_CLOCK_COUNT2_H,w
	rnz
	
	; SPI_MSG - Chip select & Message type nybble: c000mmmm
	; AARGB1  - Value to transmit byte
	movlw	b'10000110'
	movwf	SPI_MSG
	movfw	LFO2_SYNC	; Transmit LFO2 Sync Start
	movwf	AARGB1
	call	TransmitSPI
	
	clrf	MIDI_CLOCK_COUNT2_L ; Reset MIDI clock count
	clrf	MIDI_CLOCK_COUNT2_H
	
	return
	
	
;-----------------------------------------------------------------------------------------
	
	
; SPI_MSG - Chip select & Message type nybble: c000mmmm
; AARGB1  - Value to transmit byte
TransmitSPI
	movfw	SPI_MSG
	movwf	SSPBUF		; Transmit byte1
	
	btfss	PIR1,SSPIF	; Poll transmit complete flag
	goto	$-1
	bcf	PIR1,SSPIF	; Clear flag
	
	movfw	AARGB1
	movwf	SSPBUF		; Transmit byte2
	
	btfss	PIR1,SSPIF	; Poll transmit complete flag
	goto	$-1
	bcf	PIR1,SSPIF	; Clear flag
	
	return
	
	
;-----------------------------------------------------------------------------------------
	
	
; RAM Bank 0
ProcessMIDI_ControlChange
	call	PointToNextProcessBufferByte	; Get and save controller number
	movfw	MIDI_MSG_PROCESS_PTR
	movwf	FSR
	movfw	INDF
	movwf	MTEMP2
	call	PointToNextProcessBufferByte	; Get and save controller value
	movfw	MIDI_MSG_PROCESS_PTR
	movwf	FSR
	movfw	INDF
	movwf	MTEMP4
	
	; Check MIDI channel
	movfw	MIDI_CHANNEL
	subwf	MTEMP1,w
	rnz
	
	bcf	MIDI_FLAGS,ODD_VALUE
	btfsc	MTEMP2,0
	bsf	MIDI_FLAGS,ODD_VALUE
	
	movfw	MTEMP2
	sublw	D'16'
	jz	Change_DELAY_TIME_1
	
	movfw	MTEMP2
	sublw	D'17'
	jz	Change_DELAY_TIME_1
	
	movfw	MTEMP2
	sublw	D'18'
	jz	Change_DELAY_TIME_2
	
	movfw	MTEMP2
	sublw	D'19'
	jz	Change_DELAY_TIME_2
	
	movfw	MTEMP2
	sublw	D'20'
	jz	Change_DELAY_TIME_3
	
	movfw	MTEMP2
	sublw	D'21'
	jz	Change_DELAY_TIME_3
	
	movfw	MTEMP2
	sublw	D'22'
	jz	Change_DELAY_TIME_4
	
	movfw	MTEMP2
	sublw	D'23'
	jz	Change_DELAY_TIME_4
	
	movfw	MTEMP2
	sublw	D'24'
	jz	Change_LFO1_RATE
	
	movfw	MTEMP2
	sublw	D'25'
	jz	Change_LFO1_RATE
	
	movfw	MTEMP2
	sublw	D'26'
	jz	Change_LFO2_RATE
	
	movfw	MTEMP2
	sublw	D'27'
	jz	Change_LFO2_RATE
	
	movfw	MTEMP2
	sublw	D'28'
	jz	Change_LFO1_DEPTH
	
	movfw	MTEMP2
	sublw	D'29'
	jz	Change_LFO1_DEPTH
	
	movfw	MTEMP2
	sublw	D'30'
	jz	Change_LFO2_DEPTH
	
	movfw	MTEMP2
	sublw	D'31'
	jz	Change_LFO2_DEPTH
	
	movfw	MTEMP2
	sublw	D'46'
	jz	Change_LFO1_PHASE
	
	movfw	MTEMP2
	sublw	D'47'
	jz	Change_LFO2_PHASE
	
	movfw	MTEMP2
	sublw	D'48'
	jz	Change_LFO1_WAVE
	
	movfw	MTEMP2
	sublw	D'49'
	jz	Change_LFO2_WAVE
	
	movfw	MTEMP2
	sublw	D'50'
	jz	Change_LFO1_SYNC
	
	movfw	MTEMP2
	sublw	D'51'
	jz	Change_LFO2_SYNC
	
	movfw	MTEMP2
	sublw	D'52'
	jz	Change_DRY_MIX_1
	
	movfw	MTEMP2
	sublw	D'53'
	jz	Change_DRY_MIX_2
	
	movfw	MTEMP2
	sublw	D'54'
	jz	Change_DRY_MIX_3
	
	movfw	MTEMP2
	sublw	D'55'
	jz	Change_DRY_MIX_4
	
	movfw	MTEMP2
	sublw	D'56'
	jz	Change_FEEDBACK_1
	
	movfw	MTEMP2
	sublw	D'57'
	jz	Change_FEEDBACK_2
	
	movfw	MTEMP2
	sublw	D'58'
	jz	Change_FEEDBACK_3
	
	movfw	MTEMP2
	sublw	D'59'
	jz	Change_FEEDBACK_4
	
	movfw	MTEMP2
	sublw	D'80'
	jz	Change_RESAMPLING_MODE_12
	
	movfw	MTEMP2
	sublw	D'81'
	jz	Change_RESAMPLING_MODE_34
	
	movfw	MTEMP2
	sublw	D'60'
	jz	Check_DoubleDelayMode12
	
	movfw	MTEMP2
	sublw	D'61'
	jz	Check_DoubleDelayMode34
	
	return
	
	
Change_DELAY_TIME_1
	; Double value
	bcf	STATUS,C
	rlf	MTEMP4
	
	; If odd CC number, add 1
	btfsc	MIDI_FLAGS,ODD_VALUE
	incf	MTEMP4
	
Check_DELAY_TIME_1
	; If > 252, set to 252
	movfw	MTEMP4
	addlw	0xFF-D'252'
	jnc	$+3
	movlw	D'252'
	movwf	MTEMP4
	
	movfw	MTEMP4
	movwf	DELAY_TIME_1
	movwf	AARGB1
	bsf	PARAMS_CHANGED,bDELAY_TIME_1
	; MTEMP3 - MIDI CC number
	; MTEMP4 - Parameter value
	movlw	D'16'
	movwf	MTEMP3
	call	SendMIDIMessage
	; SPI_MSG - Chip select & Message type nybble: c000mmmm
	; AARGB1  - Value to transmit byte
	movlw	b'00000000'
	movwf	SPI_MSG
	call	TransmitSPI
	
	; AARGB1 - Value to display
	goto	DisplayDelayTime
	
	
Change_DELAY_TIME_2
	; Double value
	bcf	STATUS,C
	rlf	MTEMP4
	
	; If odd CC number, add 1
	btfsc	MIDI_FLAGS,ODD_VALUE
	incf	MTEMP4
	
Check_DELAY_TIME_2
	; If > 252, set to 252
	movfw	MTEMP4
	addlw	0xFF-D'252'
	jnc	$+3
	movlw	D'252'
	movwf	MTEMP4
	
	movfw	MTEMP4
	movwf	DELAY_TIME_2
	movwf	AARGB1
	bsf	PARAMS_CHANGED,bDELAY_TIME_2
	; MTEMP3 - MIDI CC number
	; MTEMP4 - Parameter value
	movlw	D'18'
	movwf	MTEMP3
	call	SendMIDIMessage
	; SPI_MSG - Chip select & Message type nybble: c000mmmm
	; AARGB1  - Value to transmit byte
	movlw	b'00000001'
	movwf	SPI_MSG
	call	TransmitSPI
	
	; AARGB1 - Value to display
	goto	DisplayDelayTime
	
	
Change_DELAY_TIME_3
	; Double value
	bcf	STATUS,C
	rlf	MTEMP4
	
	; If odd CC number, add 1
	btfsc	MIDI_FLAGS,ODD_VALUE
	incf	MTEMP4
	
Check_DELAY_TIME_3
	; If > 252, set to 252
	movfw	MTEMP4
	addlw	0xFF-D'252'
	jnc	$+3
	movlw	D'252'
	movwf	MTEMP4
	
	movfw	MTEMP4
	movwf	DELAY_TIME_3
	movwf	AARGB1
	bsf	PARAMS_CHANGED,bDELAY_TIME_3
	; MTEMP3 - MIDI CC number
	; MTEMP4 - Parameter value
	movlw	D'20'
	movwf	MTEMP3
	call	SendMIDIMessage
	; SPI_MSG - Chip select & Message type nybble: c000mmmm
	; AARGB1  - Value to transmit byte
	movlw	b'10000000'
	movwf	SPI_MSG
	call	TransmitSPI
	
	; AARGB1 - Value to display
	goto	DisplayDelayTime
	
	
Change_DELAY_TIME_4
	; Double value
	bcf	STATUS,C
	rlf	MTEMP4
	
	; If odd CC number, add 1
	btfsc	MIDI_FLAGS,ODD_VALUE
	incf	MTEMP4
	
Check_DELAY_TIME_4
	; If > 252, set to 252
	movfw	MTEMP4
	addlw	0xFF-D'252'
	jnc	$+3
	movlw	D'252'
	movwf	MTEMP4
	
	movfw	MTEMP4
	movwf	DELAY_TIME_4
	movwf	AARGB1
	bsf	PARAMS_CHANGED,bDELAY_TIME_4
	; MTEMP3 - MIDI CC number
	; MTEMP4 - Parameter value
	movlw	D'22'
	movwf	MTEMP3
	call	SendMIDIMessage
	; SPI_MSG - Chip select & Message type nybble: c000mmmm
	; AARGB1  - Value to transmit byte
	movlw	b'10000001'
	movwf	SPI_MSG
	call	TransmitSPI
	
	; AARGB1 - Value to display
	goto	DisplayDelayTime
	
	
Change_LFO1_RATE
	; Double value
	bcf	STATUS,C
	rlf	MTEMP4
	
	; If odd CC number, add 1
	btfsc	MIDI_FLAGS,ODD_VALUE
	incf	MTEMP4
	
Check_LFO1_RATE
	movfw	MTEMP4
	movwf	LFO1_RATE
	movwf	AARGB1
	bsf	PARAMS_CHANGED,bLFO1_RATE
	; MTEMP3 - MIDI CC number
	; MTEMP4 - Parameter value
	movlw	D'24'
	movwf	MTEMP3
	call	SendMIDIMessage
	; SPI_MSG - Chip select & Message type nybble: c000mmmm
	; AARGB1  - Value to transmit byte
	movlw	b'00000010'
	movwf	SPI_MSG
	call	TransmitSPI
	
	; AARGB1 - Value to display
	goto	DisplayNumber
	
	
Change_LFO2_RATE
	; Double value
	bcf	STATUS,C
	rlf	MTEMP4
	
	; If odd CC number, add 1
	btfsc	MIDI_FLAGS,ODD_VALUE
	incf	MTEMP4
	
Check_LFO2_RATE
	movfw	MTEMP4
	movwf	LFO2_RATE
	movwf	AARGB1
	bsf	PARAMS_CHANGED,bLFO2_RATE
	; MTEMP3 - MIDI CC number
	; MTEMP4 - Parameter value
	movlw	D'26'
	movwf	MTEMP3
	call	SendMIDIMessage
	; SPI_MSG - Chip select & Message type nybble: c000mmmm
	; AARGB1  - Value to transmit byte
	movlw	b'10000010'
	movwf	SPI_MSG
	call	TransmitSPI
	
	; AARGB1 - Value to display
	goto	DisplayNumber
	
	
Change_LFO1_DEPTH
	; Double value
	bcf	STATUS,C
	rlf	MTEMP4
	
	; If odd CC number, add 1
	btfsc	MIDI_FLAGS,ODD_VALUE
	incf	MTEMP4
	
Check_LFO1_DEPTH
	; If > 200, set to 200
	movfw	MTEMP4
	addlw	0xFF-D'200'
	jnc	$+3
	movlw	D'200'
	movwf	MTEMP4
	
	movfw	MTEMP4
	movwf	LFO1_DEPTH
	movwf	AARGB1
	bsf	PARAMS_CHANGED,bLFO1_DEPTH
	; MTEMP3 - MIDI CC number
	; MTEMP4 - Parameter value
	movlw	D'28'
	movwf	MTEMP3
	call	SendMIDIMessage
	; SPI_MSG - Chip select & Message type nybble: c000mmmm
	; AARGB1  - Value to transmit byte
	movlw	b'00000011'
	movwf	SPI_MSG
	call	TransmitSPI
	
	; AARGB1 - Value to display
	goto	DisplayNumber
	
	
Change_LFO2_DEPTH
	; Double value
	bcf	STATUS,C
	rlf	MTEMP4
	
	; If odd CC number, add 1
	btfsc	MIDI_FLAGS,ODD_VALUE
	incf	MTEMP4
	
Check_LFO2_DEPTH
	; If > 200, set to 200
	movfw	MTEMP4
	addlw	0xFF-D'200'
	jnc	$+3
	movlw	D'200'
	movwf	MTEMP4
	
	movfw	MTEMP4
	movwf	LFO2_DEPTH
	movwf	AARGB1
	bsf	PARAMS_CHANGED2,bLFO2_DEPTH
	; MTEMP3 - MIDI CC number
	; MTEMP4 - Parameter value
	movlw	D'30'
	movwf	MTEMP3
	call	SendMIDIMessage
	; SPI_MSG - Chip select & Message type nybble: c000mmmm
	; AARGB1  - Value to transmit byte
	movlw	b'10000011'
	movwf	SPI_MSG
	call	TransmitSPI
	
	; AARGB1 - Value to display
	goto	DisplayNumber
	
	
Change_LFO1_PHASE
Check_LFO1_PHASE
	; If > 110, set to 110
	movfw	MTEMP4
	addlw	0xFF-D'110'
	jnc	$+3
	movlw	D'110'
	movwf	MTEMP4
	
	movfw	MTEMP4
	movwf	LFO1_PHASE
	movwf	AARGB1
	bsf	PARAMS_CHANGED2,bLFO1_PHASE
	; MTEMP3 - MIDI CC number
	; MTEMP4 - Parameter value
	movlw	D'46'
	movwf	MTEMP3
	call	SendMIDIMessage
	; SPI_MSG - Chip select & Message type nybble: c000mmmm
	; AARGB1  - Value to transmit byte
	movlw	b'00000100'
	movwf	SPI_MSG
	call	TransmitSPI
	
	; AARGB1 - Value to display
	goto	DisplayPhase
	
	
Change_LFO2_PHASE
Check_LFO2_PHASE
	; If > 110, set to 110
	movfw	MTEMP4
	addlw	0xFF-D'110'
	jnc	$+3
	movlw	D'110'
	movwf	MTEMP4
	
	movfw	MTEMP4
	movwf	LFO2_PHASE
	movwf	AARGB1
	bsf	PARAMS_CHANGED2,bLFO2_PHASE
	; MTEMP3 - MIDI CC number
	; MTEMP4 - Parameter value
	movlw	D'47'
	movwf	MTEMP3
	call	SendMIDIMessage
	; SPI_MSG - Chip select & Message type nybble: c000mmmm
	; AARGB1  - Value to transmit byte
	movlw	b'10000100'
	movwf	SPI_MSG
	call	TransmitSPI
	
	; AARGB1 - Value to display
	goto	DisplayPhase
	
	
Change_LFO1_WAVE
	; If > 4, set to 4
	movfw	MTEMP4
	addlw	0xFF-D'4'
	jnc	$+3
	movlw	D'4'
	movwf	MTEMP4

	movfw	MTEMP4
	movwf	LFO1_WAVE
	movwf	AARGB1
	bsf	PARAMS_CHANGED2,bLFO1_WAVE
	; MTEMP3 - MIDI CC number
	; MTEMP4 - Parameter value
	movlw	D'48'
	movwf	MTEMP3
	call	SendMIDIMessage
	; SPI_MSG - Chip select & Message type nybble: c000mmmm
	; AARGB1  - Value to transmit byte
	movlw	b'00000101'
	movwf	SPI_MSG
	call	TransmitSPI
	
	; Write text to LED
	movlw	low (LEDText_Sin - LEDText_Base)
	addwf	AARGB1,w
	addwf	AARGB1,w
	addwf	AARGB1,w
	movwf	MTEMP1
	goto	WriteTextToLED
	
	
Change_LFO2_WAVE
	; If > 4, set to 4
	movfw	MTEMP4
	addlw	0xFF-D'4'
	jnc	$+3
	movlw	D'4'
	movwf	MTEMP4

	movfw	MTEMP4
	movwf	LFO2_WAVE
	movwf	AARGB1
	bsf	PARAMS_CHANGED2,bLFO2_WAVE
	; MTEMP3 - MIDI CC number
	; MTEMP4 - Parameter value
	movlw	D'49'
	movwf	MTEMP3
	call	SendMIDIMessage
	; SPI_MSG - Chip select & Message type nybble: c000mmmm
	; AARGB1  - Value to transmit byte
	movlw	b'10000101'
	movwf	SPI_MSG
	call	TransmitSPI
	
	; Write text to LED
	movlw	low (LEDText_Sin - LEDText_Base)
	addwf	AARGB1,w
	addwf	AARGB1,w
	addwf	AARGB1,w
	movwf	MTEMP1
	goto	WriteTextToLED
	
		
Change_LFO1_SYNC
Check_LFO1_SYNC
	; If > 13, set to 13
	movfw	MTEMP4
	addlw	0xFF-D'13'
	jnc	$+3
	movlw	D'13'
	movwf	MTEMP4
	
	movlw	high LookupLFOSync_H
	movwf	PCLATH
	movfw	MTEMP4
	call	LookupLFOSync_H
	movwf	MIDI_CLOCK_PERIOD1_H
	movfw	MTEMP4
	call	LookupLFOSync_L
	movwf	MIDI_CLOCK_PERIOD1_L
	
	movfw	MTEMP4
	movwf	LFO1_SYNC
	movwf	AARGB1
	bsf	PARAMS_CHANGED2,bLFO1_SYNC
	; MTEMP3 - MIDI CC number
	; MTEMP4 - Parameter value
	movlw	D'50'
	movwf	MTEMP3
	call	SendMIDIMessage
	; SPI_MSG - Chip select & Message type nybble: c000mmmm
	; AARGB1  - Value to transmit byte
	movlw	b'00000110'
	movwf	SPI_MSG
	call	TransmitSPI
	
	; AARGB1 - Value to display
	goto	DisplaySync
	
		
Change_LFO2_SYNC
Check_LFO2_SYNC
	; If > 13, set to 13
	movfw	MTEMP4
	addlw	0xFF-D'13'
	jnc	$+3
	movlw	D'13'
	movwf	MTEMP4
	
	movlw	high LookupLFOSync_H
	movwf	PCLATH
	movfw	MTEMP4
	call	LookupLFOSync_H
	movwf	MIDI_CLOCK_PERIOD2_H
	movfw	MTEMP4
	call	LookupLFOSync_L
	movwf	MIDI_CLOCK_PERIOD2_L
	
	movfw	MTEMP4
	movwf	LFO2_SYNC
	movwf	AARGB1
	bsf	PARAMS_CHANGED2,bLFO2_SYNC
	; MTEMP3 - MIDI CC number
	; MTEMP4 - Parameter value
	movlw	D'51'
	movwf	MTEMP3
	call	SendMIDIMessage
	; SPI_MSG - Chip select & Message type nybble: c000mmmm
	; AARGB1  - Value to transmit byte
	movlw	b'10000110'
	movwf	SPI_MSG
	call	TransmitSPI
	
	; AARGB1 - Value to display
	goto	DisplaySync
	
		
Change_RESAMPLING_MODE_12
Check_RESAMPLING_MODE_12
	; >= 64 ?
	btfsc	MTEMP4,6
	goto	Check_RESAMPLING_MODE_12_On
	
Check_RESAMPLING_MODE_12_Off
	bcf	RESAMPLING_MODE,DELAY_12
	bsf	PARAMS_CHANGED2,bRESAMPLING_MODE
	
	; MTEMP3 - MIDI CC number
	; MTEMP4 - Parameter value
	movlw	D'80'
	movwf	MTEMP3
	clrf	MTEMP4
	call	SendMIDIMessage
	
	; SPI_MSG - Chip select & Message type nybble: c000mmmm
	; AARGB1  - Value to transmit byte
	movlw	b'00000111'
	movwf	SPI_MSG
	clrf	AARGB1
	call	TransmitSPI
	
	; Write text to LED
	movlw	low (LEDText_rof - LEDText_Base)
	movwf	MTEMP1
	goto	WriteTextToLED
	
Check_RESAMPLING_MODE_12_On
	bsf	RESAMPLING_MODE,DELAY_12
	bsf	PARAMS_CHANGED2,bRESAMPLING_MODE
	
	; MTEMP3 - MIDI CC number
	; MTEMP4 - Parameter value
	movlw	D'80'
	movwf	MTEMP3
	movlw	D'127'
	movwf	MTEMP4
	call	SendMIDIMessage
	
	; SPI_MSG - Chip select & Message type nybble: c000mmmm
	; AARGB1  - Value to transmit byte
	movlw	b'00000111'
	movwf	SPI_MSG
	clrf	AARGB1
	incf	AARGB1
	call	TransmitSPI
	
	; Write text to LED
	movlw	low (LEDText_ron - LEDText_Base)
	movwf	MTEMP1
	goto	WriteTextToLED
	
		
Change_RESAMPLING_MODE_34
Check_RESAMPLING_MODE_34
	; >= 64 ?
	btfsc	MTEMP4,6
	goto	Check_RESAMPLING_MODE_34_On
	
Check_RESAMPLING_MODE_34_Off
	bcf	RESAMPLING_MODE,DELAY_34
	bsf	PARAMS_CHANGED2,bRESAMPLING_MODE
	
	; MTEMP3 - MIDI CC number
	; MTEMP4 - Parameter value
	movlw	D'81'
	movwf	MTEMP3
	clrf	MTEMP4
	call	SendMIDIMessage
	
	; SPI_MSG - Chip select & Message type nybble: c000mmmm
	; AARGB1  - Value to transmit byte
	movlw	b'10000111'
	movwf	SPI_MSG
	clrf	AARGB1
	call	TransmitSPI
	
	; Write text to LED
	movlw	low (LEDText_rof - LEDText_Base)
	movwf	MTEMP1
	goto	WriteTextToLED
	
Check_RESAMPLING_MODE_34_On
	bsf	RESAMPLING_MODE,DELAY_34
	bsf	PARAMS_CHANGED2,bRESAMPLING_MODE
	
	; MTEMP3 - MIDI CC number
	; MTEMP4 - Parameter value
	movlw	D'81'
	movwf	MTEMP3
	movlw	D'127'
	movwf	MTEMP4
	call	SendMIDIMessage
	
	; SPI_MSG - Chip select & Message type nybble: c000mmmm
	; AARGB1  - Value to transmit byte
	movlw	b'10000111'
	movwf	SPI_MSG
	clrf	AARGB1
	incf	AARGB1
	call	TransmitSPI
	
	; Write text to LED
	movlw	low (LEDText_ron - LEDText_Base)
	movwf	MTEMP1
	goto	WriteTextToLED
	
	
Check_DoubleDelayMode12
	; If > 2, set to 2
	movfw	MTEMP4
	addlw	0xFF-D'2'
	jnc	$+3
	movlw	D'2'
	movwf	MTEMP4
	
	bcf	RESAMPLING_MODE, HALF_SAMPLING_RATE_12
	bcf	RESAMPLING_MODE, SINGLE_INPUT_12
	btfsc	MTEMP4,0
	bsf	RESAMPLING_MODE, HALF_SAMPLING_RATE_12
	btfsc	MTEMP4,1
	bsf	RESAMPLING_MODE, SINGLE_INPUT_12
	
	movfw	MTEMP4
	movwf	AARGB1
	bsf	PARAMS_CHANGED2,bRESAMPLING_MODE
	; MTEMP3 - MIDI CC number
	; MTEMP4 - Parameter value
	movlw	D'60'
	movwf	MTEMP3
	call	SendMIDIMessage
	; SPI_MSG - Chip select & Message type nybble: c000mmmm
	; AARGB1  - Value to transmit byte
	movlw	b'00001110'
	movwf	SPI_MSG
	call	TransmitSPI
	
	; AARGB1 - Value to display
	goto	DisplayDoubleDelayMode
	
	
Check_DoubleDelayMode34
	; If > 2, set to 2
	movfw	MTEMP4
	addlw	0xFF-D'2'
	jnc	$+3
	movlw	D'2'
	movwf	MTEMP4
	
	bcf	RESAMPLING_MODE, HALF_SAMPLING_RATE_34
	bcf	RESAMPLING_MODE, SINGLE_INPUT_34
	btfsc	MTEMP4,0
	bsf	RESAMPLING_MODE, HALF_SAMPLING_RATE_34
	btfsc	MTEMP4,1
	bsf	RESAMPLING_MODE, SINGLE_INPUT_34
	
	movfw	MTEMP4
	movwf	AARGB1
	bsf	PARAMS_CHANGED2,bRESAMPLING_MODE
	; MTEMP3 - MIDI CC number
	; MTEMP4 - Parameter value
	movlw	D'61'
	movwf	MTEMP3
	call	SendMIDIMessage
	; SPI_MSG - Chip select & Message type nybble: c000mmmm
	; AARGB1  - Value to transmit byte
	movlw	b'10001110'
	movwf	SPI_MSG
	call	TransmitSPI
	
	; AARGB1 - Value to display
	goto	DisplayDoubleDelayMode

		
Change_DRY_MIX_1
Check_DRY_MIX_1
	; If > 99, set to 99
	movfw	MTEMP4
	addlw	0xFF-D'99'
	jnc	$+3
	movlw	D'99'
	movwf	MTEMP4
	
	movfw	MTEMP4
	movwf	DRY_MIX_1
	movwf	AARGB1
	bsf	PARAMS_CHANGED3,bDRY_MIX_1
	; MTEMP3 - MIDI CC number
	; MTEMP4 - Parameter value
	movlw	D'52'
	movwf	MTEMP3
	call	SendMIDIMessage
	; SPI_MSG - Chip select & Message type nybble: c000mmmm
	; AARGB1  - Value to transmit byte
	movlw	b'00001000'
	movwf	SPI_MSG
	call	TransmitSPI
	
	; AARGB1 - Value to display
	goto	DisplayDryMix
	
		
Change_DRY_MIX_2
Check_DRY_MIX_2
	; If > 99, set to 99
	movfw	MTEMP4
	addlw	0xFF-D'99'
	jnc	$+3
	movlw	D'99'
	movwf	MTEMP4
	
	movfw	MTEMP4
	movwf	DRY_MIX_2
	movwf	AARGB1
	bsf	PARAMS_CHANGED3,bDRY_MIX_2
	; MTEMP3 - MIDI CC number
	; MTEMP4 - Parameter value
	movlw	D'53'
	movwf	MTEMP3
	call	SendMIDIMessage
	; SPI_MSG - Chip select & Message type nybble: c000mmmm
	; AARGB1  - Value to transmit byte
	movlw	b'00001001'
	movwf	SPI_MSG
	call	TransmitSPI
	
	; AARGB1 - Value to display
	goto	DisplayDryMix
	
		
Change_DRY_MIX_3
Check_DRY_MIX_3
	; If > 99, set to 99
	movfw	MTEMP4
	addlw	0xFF-D'99'
	jnc	$+3
	movlw	D'99'
	movwf	MTEMP4
	
	movfw	MTEMP4
	movwf	DRY_MIX_3
	movwf	AARGB1
	bsf	PARAMS_CHANGED3,bDRY_MIX_3
	; MTEMP3 - MIDI CC number
	; MTEMP4 - Parameter value
	movlw	D'54'
	movwf	MTEMP3
	call	SendMIDIMessage
	; SPI_MSG - Chip select & Message type nybble: c000mmmm
	; AARGB1  - Value to transmit byte
	movlw	b'10001000'
	movwf	SPI_MSG
	call	TransmitSPI
	
	; AARGB1 - Value to display
	goto	DisplayDryMix
	
		
Change_DRY_MIX_4
Check_DRY_MIX_4
	; If > 99, set to 99
	movfw	MTEMP4
	addlw	0xFF-D'99'
	jnc	$+3
	movlw	D'99'
	movwf	MTEMP4
	
	movfw	MTEMP4
	movwf	DRY_MIX_4
	movwf	AARGB1
	bsf	PARAMS_CHANGED3,bDRY_MIX_4
	; MTEMP3 - MIDI CC number
	; MTEMP4 - Parameter value
	movlw	D'55'
	movwf	MTEMP3
	call	SendMIDIMessage
	; SPI_MSG - Chip select & Message type nybble: c000mmmm
	; AARGB1  - Value to transmit byte
	movlw	b'10001001'
	movwf	SPI_MSG
	call	TransmitSPI
	
	; AARGB1 - Value to display
	goto	DisplayDryMix
	
		
Change_FEEDBACK_1
Check_FEEDBACK_1
	; If > 99, set to 99
	movfw	MTEMP4
	addlw	0xFF-D'99'
	jnc	$+3
	movlw	D'99'
	movwf	MTEMP4
	
	movfw	MTEMP4
	movwf	FEEDBACK_1
	movwf	AARGB1
	bsf	PARAMS_CHANGED3,bFEEDBACK_1
	; MTEMP3 - MIDI CC number
	; MTEMP4 - Parameter value
	movlw	D'56'
	movwf	MTEMP3
	call	SendMIDIMessage
	; SPI_MSG - Chip select & Message type nybble: c000mmmm
	; AARGB1  - Value to transmit byte
	movlw	b'00001010'
	movwf	SPI_MSG
	call	TransmitSPI
	
	; AARGB1 - Value to display
	goto	DisplayFeedback
	
		
Change_FEEDBACK_2
Check_FEEDBACK_2
	; If > 99, set to 99
	movfw	MTEMP4
	addlw	0xFF-D'99'
	jnc	$+3
	movlw	D'99'
	movwf	MTEMP4
	
	movfw	MTEMP4
	movwf	FEEDBACK_2
	movwf	AARGB1
	bsf	PARAMS_CHANGED3,bFEEDBACK_2
	; MTEMP3 - MIDI CC number
	; MTEMP4 - Parameter value
	movlw	D'57'
	movwf	MTEMP3
	call	SendMIDIMessage
	; SPI_MSG - Chip select & Message type nybble: c000mmmm
	; AARGB1  - Value to transmit byte
	movlw	b'00001011'
	movwf	SPI_MSG
	call	TransmitSPI
	
	; AARGB1 - Value to display
	goto	DisplayFeedback
	
		
Change_FEEDBACK_3
Check_FEEDBACK_3
	; If > 99, set to 99
	movfw	MTEMP4
	addlw	0xFF-D'99'
	jnc	$+3
	movlw	D'99'
	movwf	MTEMP4
	
	movfw	MTEMP4
	movwf	FEEDBACK_3
	movwf	AARGB1
	bsf	PARAMS_CHANGED3,bFEEDBACK_3
	; MTEMP3 - MIDI CC number
	; MTEMP4 - Parameter value
	movlw	D'58'
	movwf	MTEMP3
	call	SendMIDIMessage
	; SPI_MSG - Chip select & Message type nybble: c000mmmm
	; AARGB1  - Value to transmit byte
	movlw	b'10001010'
	movwf	SPI_MSG
	call	TransmitSPI
	
	; AARGB1 - Value to display
	goto	DisplayFeedback
	
		
Change_FEEDBACK_4
Check_FEEDBACK_4
	; If > 99, set to 99
	movfw	MTEMP4
	addlw	0xFF-D'99'
	jnc	$+3
	movlw	D'99'
	movwf	MTEMP4
	
	movfw	MTEMP4
	movwf	FEEDBACK_4
	movwf	AARGB1
	bsf	PARAMS_CHANGED3,bFEEDBACK_4
	; MTEMP3 - MIDI CC number
	; MTEMP4 - Parameter value
	movlw	D'59'
	movwf	MTEMP3
	call	SendMIDIMessage
	; SPI_MSG - Chip select & Message type nybble: c000mmmm
	; AARGB1  - Value to transmit byte
	movlw	b'10001011'
	movwf	SPI_MSG
	call	TransmitSPI
	
	; AARGB1 - Value to display
	goto	DisplayFeedback
	
	

;-----------------------------------------------------------------------------------------

; Call with number in AARGB1
; Writes Lnn to LED
DisplayDryMix
	movlw	high LEDNumbers			; Set PCLATH for jump tables
	movwf	PCLATH
	
	movlw	low (LEDText_L - LEDText_Base)	; Write 'L'
	call	GetLEDNumber
	movwf	LED_DIGIT1
	
	goto	DisplayFeedback_1


; Call with number in AARGB1
; Writes fnn to LED
DisplayFeedback
	movlw	high LEDNumbers			; Set PCLATH for jump tables
	movwf	PCLATH
	
	movlw	low (LEDText_f - LEDText_Base)	; Write 'f'
	call	GetLEDNumber
	movwf	LED_DIGIT1
	
DisplayFeedback_1
	movfw	AARGB1				; Convert AARGB1 to BCD
	call	ConvertToBcd
	movfw	AARGB1				; and display on LED
	movlw	0xFF		; If 0, digit off
	jz	$+3
	movfw	AARGB1
	call	GetLEDNumber
	movwf	LED_DIGIT2
	movfw	AARGB2
	call	GetLEDNumber
	movwf	LED_DIGIT3
	
	clrf	LED_INTERRUPT_COUNT	; Turn on display
	incf	LED_INTERRUPT_COUNT
	
	return


; Call with number in AARGB1
; Writes LFO MIDI sync to LED
DisplaySync
	movlw	low (LEDText_Off - LEDText_Base)
	addwf	AARGB1,w
	addwf	AARGB1,w
	addwf	AARGB1,w
	movwf	MTEMP1
	goto	WriteTextToLED


; Call with number in AARGB1
; Writes LFO amp offset / LFO phase offset to LED
DisplayPhase
	; == 0
	movfw	AARGB1
	jnz	DisplayPhase_1
	movlw	low (LEDText_Off - LEDText_Base)
	movwf	MTEMP1
	goto	WriteTextToLED

DisplayPhase_1
	; >= 1 && <= 19
	movfw	AARGB1
	addlw	0xFF-D'19'
	jc	DisplayPhase_2
	; 20 - value
	movfw	AARGB1
	sublw	D'20'
	; * 5
	movwf	AARGB1
	addwf	AARGB1
	addwf	AARGB1
	addwf	AARGB1
	addwf	AARGB1
	
	movlw	high LEDNumbers			; Set PCLATH for jump tables
	movwf	PCLATH
	
	movlw	low (LEDText_Minus - LEDText_Base)	; Write '-'
	call	GetLEDNumber
	movwf	LED_DIGIT1
	
	movfw	AARGB1				; Convert AARGB1 to BCD
	call	ConvertToBcd
	movfw	AARGB1				; and display on LED
	movlw	0xFF		; If 0, digit off
	jz	$+3
	movfw	AARGB1
	call	GetLEDNumber
	movwf	LED_DIGIT2
	movfw	AARGB2
	call	GetLEDNumber
	movwf	LED_DIGIT3
	
	clrf	LED_INTERRUPT_COUNT	; Turn on display
	incf	LED_INTERRUPT_COUNT
	return
	
DisplayPhase_2
	; >= 20
	; value - 20
	movlw	D'20'
	subwf	AARGB1
	; * 2
	bcf	STATUS,C
	rlf	AARGB1
	
	goto	DisplayNumber


; Call with number in AARGB1
; Writes delay time to LED
; 0->49 = 0.0->4.9 (0.1 step)
; 50->74 = 5.0->9.8 (0.2 step)
; 75->114 = 10->49 (1 step)
; 115->189 = 50->198 (2 step)
; 190->219 = 200->345 (5 step)
; 220->252 = 350->670 (10 step)
DisplayDelayTime
	; >= 220 (display 350+) ?
	movfw	AARGB1
	addlw	0x100-D'220'
	jc	DisplayDelayTime_x10ms
	
	; >= 190 (display 200+) ?
	movfw	AARGB1
	addlw	0x100-D'190'
	jc	DisplayDelayTime_x5ms
	
	; >= 115 (display 50+) ?
	movfw	AARGB1
	addlw	0x100-D'115'
	jc	DisplayDelayTime_x2ms
	
	; >= 75 (display 10+) ?
	movfw	AARGB1
	addlw	0x100-D'75'
	jc	DisplayDelayTime_x1ms
	
	; >= 50 (display 5.0 to 9.8) ?
	movfw	AARGB1
	addlw	0x100-D'50'
	jc	DisplayDelayTime_xp2ms
	
; 0->49 = 0.0->4.9 (0.1 step)
	bsf	FLAGS,SHOW_TENS
	call	DisplayNumber
	bcf	FLAGS,SHOW_TENS
	; Write "."
	bcf	LED_DIGIT2,LED_SEG_DP
	return
	
; 50->74 = 5.0->9.8 (0.2 step)
DisplayDelayTime_xp2ms
	; - 25
	movlw	D'25'
	subwf	AARGB1
	; * 2
	movfw	AARGB1
	addwf	AARGB1
	call	DisplayNumber
	; Write "."
	bcf	LED_DIGIT2,LED_SEG_DP
	return
	
; 75->114 = 10->49 (1 step)
DisplayDelayTime_x1ms
	; - 65
	movlw	D'65'
	subwf	AARGB1
	goto	DisplayNumber
	
; 115->189 = 50->198 (2 step)
DisplayDelayTime_x2ms
	; - 90
	movlw	D'90'
	subwf	AARGB1
	; * 2
	movfw	AARGB1
	addwf	AARGB1
	goto	DisplayNumber
	
; 190->219 = 200->345 (5 step)
DisplayDelayTime_x5ms
	; - 190
	movlw	D'190'
	subwf	AARGB1
	; MTEMP1 = 2
	movlw	2
	movwf	MTEMP1
	
DisplayDelayTime_x5ms_Loop
	; AARGB1 < 20 ?
	movfw	AARGB1
	addlw	0x100-D'20'
	jnc	DisplayDelayTime_x5ms_EOL
	
	; MTEMP1 ++
	incf	MTEMP1
	; AARGB1 -= 20
	movlw	D'20'
	subwf	AARGB1
	goto	DisplayDelayTime_x5ms_Loop
	
DisplayDelayTime_x5ms_EOL
	; Number of hundreds in MTEMP1

	; AARGB1 * 5
	movfw	AARGB1
	addwf	AARGB1
	addwf	AARGB1
	addwf	AARGB1
	addwf	AARGB1
	
	bsf	FLAGS,SHOW_TENS
	call	DisplayNumber
	bcf	FLAGS,SHOW_TENS
	; Write hundreds digit
	movfw	MTEMP1
	call	GetLEDNumber
	movwf	LED_DIGIT1
	return
	
; 220->252 = 350->670 (10 step)
DisplayDelayTime_x10ms
	; - 215
	movlw	D'215'
	subwf	AARGB1
	; MTEMP1 = 3
	movlw	3
	movwf	MTEMP1
	
DisplayDelayTime_x10ms_Loop
	; AARGB1 < 10 ?
	movfw	AARGB1
	addlw	0x100-D'10'
	jnc	DisplayDelayTime_x10ms_EOL
	
	; MTEMP1 ++
	incf	MTEMP1
	; AARGB1 -= 10
	movlw	D'10'
	subwf	AARGB1
	goto	DisplayDelayTime_x10ms_Loop
	
DisplayDelayTime_x10ms_EOL
	; Number of hundreds in MTEMP1

	; AARGB1 * 10
	movfw	AARGB1
	addwf	AARGB1
	addwf	AARGB1
	addwf	AARGB1
	addwf	AARGB1
	movfw	AARGB1
	addwf	AARGB1
	
	bsf	FLAGS,SHOW_TENS
	call	DisplayNumber
	bcf	FLAGS,SHOW_TENS
	; Write hundreds digit
	movfw	MTEMP1
	call	GetLEDNumber
	movwf	LED_DIGIT1
	return


; Call with number in AARGB1
; Writes "dt1", "dt2", "dt2."
DisplayDoubleDelayMode
	movlw	low (LEDText_dt1 - LEDText_Base)
	addwf	AARGB1,w
	addwf	AARGB1,w
	addwf	AARGB1,w
	movwf	MTEMP1
	goto	WriteTextToLED


; Call with number in AARGB1
; Writes number to LED
DisplayNumber
	movlw	high LEDNumbers	; Set PCLATH for jump tables
	movwf	PCLATH
	
	; Convert result to BCD
	movfw	AARGB1
	call	ConvertToBcd
	
	; Write to result to LED
	movfw	AARGB0
	movlw	0xFF		; If 0, digit off
	jz	DisplayNumber_1
	movfw	AARGB0
	call	GetLEDNumber
DisplayNumber_1
	movwf	LED_DIGIT1
	
	movfw	AARGB1
	jnz	DisplayNumber_2	; If not 0, jump to display digit
	btfsc	FLAGS,SHOW_TENS	; If SHOW_TENS flag set, jump to display digit
	goto	DisplayNumber_2
	movfw	AARGB0		; If digit1 also 0, digit off
	movlw	0xFF
	jz	DisplayNumber_3
	movfw	AARGB1
DisplayNumber_2
	call	GetLEDNumber
DisplayNumber_3
	movwf	LED_DIGIT2
	
	movfw	AARGB2
	call	GetLEDNumber
	movwf	LED_DIGIT3

	btfss	INTCON,TMR0IE		; If LED updates already off, don't change them here
	goto	DisplayNumber_4

	bcf	INTCON,TMR0IE		; LED updates off
	clrf	LED_INTERRUPT_COUNT	; Turn on display for 1.7s
	incf	LED_INTERRUPT_COUNT
	bsf	INTCON,TMR0IE		; LED updates on
	
	return
	
DisplayNumber_4
	clrf	LED_INTERRUPT_COUNT	; Turn on display for 1.7s
	incf	LED_INTERRUPT_COUNT
	return


;-----------------------------------------------------------------------------------------


; Call with byte in w
;
; Returns with:
; hundreds in AARGB0
; tens     in AARGB1
; ones     in AARGB2
ConvertToBcd
	movwf	AARGB2
	BinaryToBcd
	movfw	AARGB1
	andlw	b'00001111'
	movwf	AARGB2
	movlw	b'11110000'
	andwf	AARGB1
	bcf	STATUS,C
	rrf	AARGB1
	rrf	AARGB1
	rrf	AARGB1
	rrf	AARGB1
	return


;-----------------------------------------------------------------------------------------


; Writes referenced text onto LED	
; Call with offset text relative to LEDText_Base in MTEMP1
; Call only during main loop (or initialisation)
; RAM bank 0
WriteTextToLED
	movlw	high LEDNumbers	; Set PCLATH for jump tables
	movwf	PCLATH
	
	movfw	MTEMP1
	call	GetLEDNumber
	movwf	LED_DIGIT1
	incf	MTEMP1
	movfw	MTEMP1
	call	GetLEDNumber
	movwf	LED_DIGIT2
	incf	MTEMP1,w
	call	GetLEDNumber
	movwf	LED_DIGIT3
	
	clrf	LED_INTERRUPT_COUNT	; Turn on display for 1.7s
	incf	LED_INTERRUPT_COUNT
	return
	
	
;-----------------------------------------------------------------------------------------


; RAM bank 0
;
; MTEMP3 - MIDI CC number
; MTEMP4 - Parameter value
;
; Sends MIDI message
SendMIDIMessage
	btfss	MIDI_FLAGS,SEND_MIDI_CC	; Return if flag not set
	return
	bcf	MIDI_FLAGS,SEND_MIDI_CC	; Clear flag
	
	movfw	MTEMP3			; Is CC >= 46, then skip halving value
	addlw	0x100-D'46'
	jc	SendMIDIMessage_1
	
	btfss	MTEMP4,0		; If even number then skip to just halve value and send on base CC
	goto	SendMIDIMessage_HalveValue
	
	decf	MTEMP4			; If odd number then decrement before halving, and increment CC number
	incf	MTEMP3
	
SendMIDIMessage_HalveValue
	bcf	STATUS,C		; Halve CC value
	rrf	MTEMP4
	
SendMIDIMessage_1
	movfw	MIDI_CHANNEL		; Combine message code with MIDI channel
	iorlw	0xB0
	movwf	TXREG			; Transmit
	
	; Wait for byte to be transmitted
	bsf	STATUS,RP0	; RAM bank 1
	btfss	TXSTA,TRMT
	goto	$-1
	bcf	STATUS,RP0	; RAM bank 0
	
	movfw	MTEMP3			; Transmit CC number
	movwf	TXREG
	
	; Wait for byte to be transmitted
	bsf	STATUS,RP0	; RAM bank 1
	btfss	TXSTA,TRMT
	goto	$-1
	bcf	STATUS,RP0	; RAM bank 0

	movfw	MTEMP4			; Transmit CC value
	movwf	TXREG
	
	; Wait for byte to be transmitted
	bsf	STATUS,RP0	; RAM bank 1
	btfss	TXSTA,TRMT
	goto	$-1
	bcf	STATUS,RP0	; RAM bank 0
	
	return
	
	
	
;#######################################################################


	; Program bank 3
	
	org	0x1800
	
	nop
	
	
	; Last 0x100 words reserved for ICD
	org	0x1F00
	nop


;#######################################################################


	; EEPROM
	
 	org	0x2100

; 	de	0xFF		; Set EE_MEMORY_LOCATION to 'invalid'


;#######################################################################


	END			 ; directive 'end of program'


